@@ -1,5 +1,5 @@
 /* cat -- concatenate files and print on the standard output.
-   Copyright (C) 88, 90, 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 88, 90, 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,19 +28,22 @@
 #include <getopt.h>
 #include <sys/types.h>
 #ifndef _POSIX_SOURCE
-#include <sys/ioctl.h>
+# include <sys/ioctl.h>
 #endif
 #include "system.h"
 #include "error.h"
+#include "safe-read.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "cat"
+
+#define AUTHORS "Torbjorn Granlund and Richard M. Stallman"
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef max
 #define max(h,i) ((h) > (i) ? (h) : (i))
 
-char *stpcpy ();
-char *xmalloc ();
 int full_write ();
-int safe_read ();
 
 /* Name under which this program was invoked.  */
 char *program_name;
@@ -74,7 +77,7 @@ static int newlines2 = 0;
 /* Count of non-fatal error conditions.  */
 static int exit_status = 0;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -104,7 +107,13 @@ Concatenate FILE(s), or standard input, to standard output.\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+#if O_BINARY
+      printf (_("\
+\n\
+  -B, --binary             use binary writes to the console device.\n\n\
+"));
+#endif
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -279,9 +288,9 @@ cat (
 		     Irix-5 returns ENOSYS on pipes.  */
 		  if (errno == EOPNOTSUPP || errno == ENOTTY
 		      || errno == EINVAL || errno == ENODEV
-#ifdef ENOSYS
+# ifdef ENOSYS
 		      || errno == ENOSYS
-#endif
+# endif
 		      )
 		    use_fionread = 0;
 		  else
@@ -383,60 +392,76 @@ cat (
       /* If quoting, i.e. at least one of -v, -e, or -t specified,
 	 scan for chars that need conversion.  */
       if (quote)
-	for (;;)
-	  {
-	    if (ch >= 32)
-	      {
-		if (ch < 127)
-		  *bpout++ = ch;
-		else if (ch == 127)
-		  *bpout++ = '^',
-		    *bpout++ = '?';
-		else
-		  {
-		    *bpout++ = 'M',
+	{
+	  for (;;)
+	    {
+	      if (ch >= 32)
+		{
+		  if (ch < 127)
+		    *bpout++ = ch;
+		  else if (ch == 127)
+		    {
+		      *bpout++ = '^';
+		      *bpout++ = '?';
+		    }
+		  else
+		    {
+		      *bpout++ = 'M';
 		      *bpout++ = '-';
-		    if (ch >= 128 + 32)
-		      if (ch < 128 + 127)
-			*bpout++ = ch - 128;
+		      if (ch >= 128 + 32)
+			{
+			  if (ch < 128 + 127)
+			    *bpout++ = ch - 128;
+			  else
+			    {
+			      *bpout++ = '^';
+			      *bpout++ = '?';
+			    }
+			}
 		      else
-			*bpout++ = '^',
-			  *bpout++ = '?';
-		    else
-		      *bpout++ = '^',
-			*bpout++ = ch - 128 + 64;
-		  }
-	      }
-	    else if (ch == '\t' && output_tabs)
-	      *bpout++ = '\t';
-	    else if (ch == '\n')
-	      {
-		newlines = -1;
-		break;
-	      }
-	    else
-	      *bpout++ = '^',
-		*bpout++ = ch + 64;
+			{
+			  *bpout++ = '^';
+			  *bpout++ = ch - 128 + 64;
+			}
+		    }
+		}
+	      else if (ch == '\t' && output_tabs)
+		*bpout++ = '\t';
+	      else if (ch == '\n')
+		{
+		  newlines = -1;
+		  break;
+		}
+	      else
+		{
+		  *bpout++ = '^';
+		  *bpout++ = ch + 64;
+		}
 
-	    ch = *bpin++;
-	  }
+	      ch = *bpin++;
+	    }
+	}
       else
-	/* Not quoting, neither of -v, -e, or -t specified.  */
-	for (;;)
-	  {
-	    if (ch == '\t' && !output_tabs)
-	      *bpout++ = '^',
-		*bpout++ = ch + 64;
-	    else if (ch != '\n')
-	      *bpout++ = ch;
-	    else
-	      {
-		newlines = -1;
-		break;
-	      }
+	{
+	  /* Not quoting, neither of -v, -e, or -t specified.  */
+	  for (;;)
+	    {
+	      if (ch == '\t' && !output_tabs)
+		{
+		  *bpout++ = '^';
+		  *bpout++ = ch + 64;
+		}
+	      else if (ch != '\n')
+		*bpout++ = ch;
+	      else
+		{
+		  newlines = -1;
+		  break;
+		}
 
-	    ch = *bpin++;
-	  }
+	      ch = *bpin++;
+	    }
+	}
     }
 }
 
@@ -461,10 +486,10 @@ main (int argc, char **argv)
   int argind;
 
   /* Device number of the output (file or whatever).  */
-  int out_dev;
+  dev_t out_dev;
 
   /* I-node number of the output.  */
-  int out_ino;
+  ino_t out_ino;
 
   /* Nonzero if the output file should not be the same as any input file. */
   int check_redirection = 1;
@@ -481,16 +506,15 @@ main (int argc, char **argv)
   int mark_line_ends = 0;
   int quote = 0;
   int output_tabs = 1;
+#if O_BINARY
+  int binary_files  = 0;
+  int binary_output = 0;
+#endif
+  int file_open_mode = O_RDONLY;
 
 /* If nonzero, call cat, otherwise call simple_cat to do the actual work. */
   int options = 0;
 
-  /* If nonzero, display usage information and exit.  */
-  static int show_help;
-
-  /* If nonzero, print the version on standard output then exit.  */
-  static int show_version;
-
   static struct option const long_options[] =
   {
     {"number-nonblank", no_argument, NULL, 'b'},
@@ -500,8 +524,11 @@ main (int argc, char **argv)
     {"show-ends", no_argument, NULL, 'E'},
     {"show-tabs", no_argument, NULL, 'T'},
     {"show-all", no_argument, NULL, 'A'},
-    {"help", no_argument, &show_help, 1},
-    {"version", no_argument, &show_version, 1},
+#if O_BINARY
+    {"binary", no_argument, NULL, 'B'},
+#endif
+    {GETOPT_HELP_OPTION_DECL},
+    {GETOPT_VERSION_OPTION_DECL},
     {NULL, 0, NULL, 0}
   };
 
@@ -512,8 +539,13 @@ main (int argc, char **argv)
 
   /* Parse command line options.  */
 
-  while ((c = getopt_long (argc, argv, "benstuvAET", long_options, (int *) 0))
-	 != EOF)
+  while ((c = getopt_long (argc, argv,
+#if O_BINARY
+			   "benstuvABET"
+#else
+			   "benstuvAET"
+#endif
+			   , long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -564,6 +596,13 @@ main (int argc, char **argv)
 	  output_tabs = 0;
 	  break;
 
+#if O_BINARY
+	case 'B':
+	  ++options;
+	  binary_files = 1;
+	  break;
+#endif
+
 	case 'E':
 	  ++options;
 	  mark_line_ends = 1;
@@ -574,20 +613,15 @@ main (int argc, char **argv)
 	  output_tabs = 0;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (EXIT_FAILURE);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("cat (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   output_desc = 1;
 
   /* Get device, i-node number, and optimal blocksize of output.  */
@@ -616,6 +650,42 @@ main (int argc, char **argv)
 #endif
     }
 
+#if O_BINARY
+  /* We always read and write in BINARY mode, since this is the
+     best way to copy the files verbatim.  Exceptions are when
+     they request line numbering, squeezing of empty lines or
+     marking lines' ends: then we use text I/O, because otherwise
+     -b, -s and -E would surprise users on DOS/Windows where a line
+     with only CR-LF is an empty line.  (Besides, if they ask for
+     one of these options, they don't care much about the original
+     file contents anyway).  */
+  if ((!isatty (output_desc)
+       && !(numbers || squeeze_empty_lines || mark_line_ends))
+      || binary_files)
+    {
+      /* Switch stdout to BINARY mode.  */
+      binary_output = 1;
+      SET_BINARY (output_desc);
+      /* When stdout is in binary mode, make sure all input files are
+	 also read in binary mode.  */
+      file_open_mode |= O_BINARY;
+    }
+  else if (quote)
+    {
+      /* If they want to see the non-printables, let's show them
+	 those CR characters as well, so make the input binary.
+	 But keep console output in text mode, so that LF causes
+	 both CR and LF on output, and the output is readable.  */
+      file_open_mode |= O_BINARY;
+      SET_BINARY (0);
+
+      /* Setting stdin to binary switches the console device to
+	 raw I/O, which also affects stdout to console.  Undo that.  */
+      if (isatty (output_desc))
+	setmode (output_desc, O_TEXT);
+    }
+#endif
+
   /* Check if any of the input files are the same as the output file.  */
 
   /* Main loop.  */
@@ -632,10 +702,37 @@ main (int argc, char **argv)
 	{
 	  have_read_stdin = 1;
 	  input_desc = 0;
+
+#if O_BINARY
+	  /* Switch stdin to BINARY mode if needed.  */
+	  if (binary_output)
+	    {
+	      int tty_in = isatty (input_desc);
+
+	      /* If stdin is a terminal device, and it is the ONLY
+		 input file (i.e. we didn't write anything to the
+		 output yet), switch the output back to TEXT mode.
+		 This is so "cat > xyzzy" creates a DOS-style text
+		 file, like people expect.  */
+	      if (tty_in && optind <= argc)
+		setmode (output_desc, O_TEXT);
+	      else
+		{
+		  SET_BINARY (input_desc);
+# ifdef __DJGPP__
+		  /* This is DJGPP-specific.  By default, switching console
+		     to binary mode disables SIGINT.  But we want terminal
+		     reads to be interruptible.  */
+		  if (tty_in)
+		    __djgpp_set_ctrl_c (1);
+# endif
+		}
+	    }
+#endif
 	}
       else
 	{
-	  input_desc = open (infile, O_RDONLY);
+	  input_desc = open (infile, file_open_mode);
 	  if (input_desc < 0)
 	    {
 	      error (0, errno, "%s", infile);
@@ -709,7 +806,7 @@ main (int argc, char **argv)
       free (inbuf);
 
     contin:
-      if (strcmp (infile, "-") && close (input_desc) < 0)
+      if (!STREQ (infile, "-") && close (input_desc) < 0)
 	{
 	  error (0, errno, "%s", infile);
 	  exit_status = 1;
