@@ -1,5 +1,5 @@
 /* cksum -- calculate and print POSIX.2 checksums and sizes of files
-   Copyright (C) 92, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 92, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Q. Frank Xia, qx@math.columbia.edu.
    Cosmetic changes and reorganization by David MacKenzie, djm@gnu.ai.mit.edu.
@@ -42,12 +42,17 @@
 
 #include <config.h>
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "cksum"
+
+#define AUTHORS "Q. Frank Xia"
+
 #ifdef CRCTAB
 
-#include <stdio.h>
+# include <stdio.h>
 
-#define BIT(x)	( (unsigned long)1 << (x) )
-#define SBIT	BIT(31)
+# define BIT(x)	( (unsigned long)1 << (x) )
+# define SBIT	BIT(31)
 
 /* The generating polynomial is
 
@@ -56,7 +61,7 @@
 
   The i bit in GEN is set if X^i is a summand of G(X) except X^32.  */
 
-#define GEN     (BIT(26)|BIT(23)|BIT(22)|BIT(16)|BIT(12)|BIT(11)|BIT(10)\
+# define GEN     (BIT(26)|BIT(23)|BIT(22)|BIT(16)|BIT(12)|BIT(11)|BIT(10)\
                 |BIT(8) |BIT(7) |BIT(5) |BIT(4) |BIT(2) |BIT(1) |BIT(0));
 
 static unsigned long r[8];
@@ -104,28 +109,21 @@ main ()
 
 #else /* !CRCTAB */
 
-#include <stdio.h>
-#include <getopt.h>
-#include <sys/types.h>
-#include "system.h"
-#include "error.h"
+# include <stdio.h>
+# include <getopt.h>
+# include <sys/types.h>
+# include "system.h"
+# include "long-options.h"
+# include "error.h"
 
 /* Number of bytes to read at once.  */
-#define BUFLEN (1 << 16)
+# define BUFLEN (1 << 16)
 
 /* The name this program was run with.  */
 char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
@@ -202,7 +200,7 @@ cksum (char *file, int print_name)
   long bytes_read;
   register FILE *fp;
 
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     {
       fp = stdin;
       have_read_stdin = 1;
@@ -217,6 +215,9 @@ cksum (char *file, int print_name)
 	}
     }
 
+  /* Read input in BINARY mode, unless it is a console device.  */
+  SET_BINARY (fileno (fp));
+
   while ((bytes_read = fread (buf, 1, BUFLEN, fp)) > 0)
     {
       unsigned char *cp = buf;
@@ -229,12 +230,12 @@ cksum (char *file, int print_name)
   if (ferror (fp))
     {
       error (0, errno, "%s", file);
-      if (strcmp (file, "-"))
+      if (!STREQ (file, "-"))
 	fclose (fp);
       return -1;
     }
 
-  if (strcmp (file, "-") && fclose (fp) == EOF)
+  if (!STREQ (file, "-") && fclose (fp) == EOF)
     {
       error (0, errno, "%s", file);
       return -1;
@@ -257,7 +258,7 @@ cksum (char *file, int print_name)
   return 0;
 }
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -275,6 +276,7 @@ Print CRC checksum and byte counts of each FILE.\n\
   --help      display this help and exit\n\
   --version   output version information and exit\n\
 "));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -290,9 +292,12 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
+
   have_read_stdin = 0;
 
-  while ((c = getopt_long (argc, argv, "", long_options, (int *) 0)) != EOF)
+  while ((c = getopt_long (argc, argv, "", long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -304,15 +309,6 @@ main (int argc, char **argv)
 	}
     }
 
-  if (show_version)
-    {
-      printf ("cksum - %s\n", PACKAGE_VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (optind >= argc)
     {
       if (cksum ("-", 0) < 0)
