@@ -1,5 +1,5 @@
 /* cut - remove parts of lines of files
-   Copyright (C) 1984 by David M. Ihnat
+   Copyright (C) 1984, 1997, 1998, 1999 by David M. Ihnat
 
    This program is a total rewrite of the Bell Laboratories Unix(Tm)
    command of the same name, as of System V.  It contains no proprietary
@@ -61,23 +61,24 @@
 
 #include <config.h>
 
-/* Get isblank from GNU libc.  */
-#define _GNU_SOURCE
-
 #include <stdio.h>
-
-#define NDEBUG
 #include <assert.h>
-
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
 #include "error.h"
 
-#define FATAL_ERROR(s)							\
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "cut"
+
+#define AUTHORS "David Ihnat, David MacKenzie, and Jim Meyering"
+
+char *xstrdup ();
+
+#define FATAL_ERROR(Message)						\
   do									\
     {									\
-      error (0, 0, (s));						\
+      error (0, 0, (Message));						\
       usage (2);							\
     }									\
   while (0)
@@ -92,7 +93,7 @@
       if (n_rp >= n_rp_allocated)					\
 	{								\
 	  n_rp_allocated *= 2;						\
-	  (rp) = (struct range_pair *) xrealloc ((rp),			\
+	  (rp) = (struct range_pair *) xrealloc ((char *) (rp),		\
 				   n_rp_allocated * sizeof (*(rp)));	\
 	}								\
       rp[n_rp].lo = (low);						\
@@ -107,9 +108,6 @@ struct range_pair
     unsigned int hi;
   };
 
-char *xmalloc ();
-char *xrealloc ();
-
 /* This buffer is used to support the semantics of the -s option
    (or lack of same) when the specified field list includes (does
    not include) the first field.  In both of those cases, the entire
@@ -165,15 +163,16 @@ static int suppress_non_delimited;
 /* The delimeter character for field mode. */
 static int delim;
 
+/* The length of output_delimiter_string.  */
+static size_t output_delimiter_length;
+
+/* The output field separator string.  Defaults to the 1-character
+   string consisting of the input delimiter.  */
+static char *output_delimiter_string;
+
 /* Nonzero if we have ever read standard input. */
 static int have_read_stdin;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
   {"bytes", required_argument, 0, 'b'},
@@ -181,12 +180,13 @@ static struct option const longopts[] =
   {"fields", required_argument, 0, 'f'},
   {"delimiter", required_argument, 0, 'd'},
   {"only-delimited", no_argument, 0, 's'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {"output-delimiter", required_argument, 0, CHAR_MAX + 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {0, 0, 0, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -207,6 +207,8 @@ Print selected parts of lines from each FILE to standard output.\n\
   -f, --fields=LIST       output only these fields\n\
   -n                      (ignored)\n\
   -s, --only-delimited    do not print lines not containing delimiters\n\
+      --output-delimiter=STRING  use STRING as the output delimiter\n\
+                            the default is to use the input delimiter\n\
       --help              display this help and exit\n\
       --version           output version information and exit\n\
 \n\
@@ -220,7 +222,7 @@ range, or many ranges separated by commas.  Each range is one of:\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -242,7 +244,7 @@ With no FILE, or when FILE is -, read standard input.\n\
    including the null terminator), or -1 on error or EOF.  */
 
 static int
-getstr (char **lineptr, int *n, FILE *stream, char terminator)
+getstr (char **lineptr, int *n, FILE *stream, int terminator)
 {
   int nchars_avail;		/* Allocated but unused chars in *LINEPTR.  */
   char *read_pos;		/* Where we're reading into *LINEPTR. */
@@ -253,7 +255,7 @@ getstr (char **lineptr, int *n, FILE *stream, char terminator)
   if (!*lineptr)
     {
       *n = MIN_CHUNK;
-      *lineptr = xmalloc (*n);
+      *lineptr = (char *) xmalloc (*n);
       if (!*lineptr)
 	return -1;
     }
@@ -526,10 +528,16 @@ cut_fields (FILE *stream)
   unsigned int field_idx;
   int found_any_selected_field;
   int buffer_first_field;
+  int empty_input;
 
   found_any_selected_field = 0;
   field_idx = 1;
 
+  c = getc (stream);
+  empty_input = (c == EOF);
+  if (c != EOF)
+    ungetc (c, stream);
+
   /* To support the semantics of the -s flag, we may have to buffer
      all of the first field to determine whether it is `delimited.'
      But that is unnecessary if all non-delimited lines must be printed
@@ -553,7 +561,7 @@ cut_fields (FILE *stream)
 	  /* If the first field extends to the end of line (it is not
 	     delimited) and we are printing all non-delimited lines,
 	     print this one.  */
-	  if (field_1_buffer[len - 1] != delim)
+	  if ((unsigned char) field_1_buffer[len - 1] != delim)
 	    {
 	      if (suppress_non_delimited)
 		{
@@ -577,22 +585,28 @@ cut_fields (FILE *stream)
 	  ++field_idx;
 	}
 
-      if (print_kth (field_idx))
+      if (c != EOF)
 	{
-	  if (found_any_selected_field)
-	    putchar (delim);
-	  found_any_selected_field = 1;
+	  if (print_kth (field_idx))
+	    {
+	      if (found_any_selected_field)
+		{
+		  fwrite (output_delimiter_string, sizeof (char),
+			  output_delimiter_length, stdout);
+		}
+	      found_any_selected_field = 1;
 
-	  while ((c = getc (stream)) != delim && c != '\n' && c != EOF)
-	    {
-	      putchar (c);
+	      while ((c = getc (stream)) != delim && c != '\n' && c != EOF)
+		{
+		  putchar (c);
+		}
 	    }
-	}
-      else
-	{
-	  while ((c = getc (stream)) != delim && c != '\n' && c != EOF)
+	  else
 	    {
-	      /* Empty.  */
+	      while ((c = getc (stream)) != delim && c != '\n' && c != EOF)
+		{
+		  /* Empty.  */
+		}
 	    }
 	}
 
@@ -611,7 +625,7 @@ cut_fields (FILE *stream)
       else if (c == '\n' || c == EOF)
 	{
 	  if (found_any_selected_field
-	      || !(suppress_non_delimited && field_idx == 1))
+	      || (!empty_input && !(suppress_non_delimited && field_idx == 1)))
 	    putchar ('\n');
 	  if (c == EOF)
 	    break;
@@ -638,7 +652,7 @@ cut_file (char *file)
 {
   FILE *stream;
 
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     {
       have_read_stdin = 1;
       stream = stdin;
@@ -660,7 +674,7 @@ cut_file (char *file)
       error (0, errno, "%s", file);
       return 1;
     }
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     clearerr (stream);		/* Also clear EOF. */
   else if (fclose (stream) == EOF)
     {
@@ -674,6 +688,7 @@ int
 main (int argc, char **argv)
 {
   int optc, exit_status = 0;
+  int delim_specified = 0;
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
@@ -688,8 +703,7 @@ main (int argc, char **argv)
   delim = '\0';
   have_read_stdin = 0;
 
-  while ((optc = getopt_long (argc, argv, "b:c:d:f:ns", longopts, (int *) 0))
-	 != EOF)
+  while ((optc = getopt_long (argc, argv, "b:c:d:f:ns", longopts, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -720,7 +734,16 @@ main (int argc, char **argv)
 	  /* Interpret -d '' to mean `use the NUL byte as the delimiter.'  */
 	  if (optarg[0] != '\0' && optarg[1] != '\0')
 	    FATAL_ERROR (_("the delimiter must be a single character"));
-	  delim = optarg[0];
+	  delim = (unsigned char) optarg[0];
+	  delim_specified = 1;
+	  break;
+
+	case CHAR_MAX + 1:
+	  /* Interpret --output-delimiter='' to mean
+	     `use the NUL byte as the delimiter.'  */
+	  output_delimiter_length = (optarg[0] == '\0'
+				     ? 1 : strlen (optarg));
+	  output_delimiter_string = xstrdup (optarg);
 	  break;
 
 	case 'n':
@@ -730,20 +753,15 @@ main (int argc, char **argv)
 	  suppress_non_delimited = 1;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (2);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("cut (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (operating_mode == undefined_mode)
     FATAL_ERROR (_("you must specify a list of bytes, characters, or fields"));
 
@@ -754,9 +772,18 @@ main (int argc, char **argv)
     FATAL_ERROR (_("suppressing non-delimited lines makes sense\n\
 \tonly when operating on fields"));
 
-  if (delim == '\0')
+  if (!delim_specified)
     delim = '\t';
 
+  if (output_delimiter_string == NULL)
+    {
+      static char dummy[2];
+      dummy[0] = delim;
+      dummy[1] = '\0';
+      output_delimiter_string = dummy;
+      output_delimiter_length = 1;
+    }
+
   if (optind == argc)
     exit_status |= cut_file ("-");
   else
