@@ -1,5 +1,5 @@
 /* expand - convert tabs to spaces
-   Copyright (C) 89, 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 89, 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,15 +35,17 @@
 
 #include <config.h>
 
-/* Get isblank from GNU libc.  */
-#define _GNU_SOURCE
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
 #include "error.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "expand"
+
+#define AUTHORS "David MacKenzie"
+
 /* The number of bytes added at a time to the amount of memory
    allocated for the output line. */
 #define OUTPUT_BLOCK 256
@@ -52,9 +54,6 @@
    allocated for the list of tabstops. */
 #define TABLIST_BLOCK 256
 
-char *xmalloc ();
-char *xrealloc ();
-
 /* The name this program was run with. */
 char *program_name;
 
@@ -89,22 +88,16 @@ static int have_read_stdin;
 /* Status to return to the system. */
 static int exit_status;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
   {"tabs", required_argument, NULL, 't'},
   {"initial", no_argument, NULL, 'i'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -128,7 +121,7 @@ With no FILE, or when FILE is -, read standard input.\n\
 \n\
 Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -142,8 +135,9 @@ add_tabstop (int tabval)
   if (tabval == -1)
     return;
   if (first_free_tab % TABLIST_BLOCK == 0)
-    tab_list = (int *) xrealloc (tab_list, first_free_tab
-				 + TABLIST_BLOCK * sizeof (tab_list[0]));
+    tab_list = (int *) xrealloc ((char *) tab_list,
+				 (first_free_tab
+				  + TABLIST_BLOCK * sizeof (tab_list[0])));
   tab_list[first_free_tab++] = tabval;
 }
 
@@ -257,6 +251,10 @@ expand (void)
   fp = next_file ((FILE *) NULL);
   if (fp == NULL)
     return;
+
+  /* Binary I/O will preserve the original EOL style (DOS/Unix) of files.  */
+  SET_BINARY2 (fileno (fp), STDOUT_FILENO);
+
   for (;;)
     {
       c = getc (fp);
@@ -266,7 +264,10 @@ expand (void)
 	  if (fp == NULL)
 	    break;		/* No more files. */
 	  else
-	    continue;
+	    {
+	      SET_BINARY2 (fileno (fp), STDOUT_FILENO);
+	      continue;
+	    }
 	}
 
       if (c == '\n')
@@ -338,8 +339,7 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while ((c = getopt_long (argc, argv, "it:,0123456789", longopts, (int *) 0))
-	 != EOF)
+  while ((c = getopt_long (argc, argv, "it:,0123456789", longopts, NULL)) != -1)
     {
       switch (c)
 	{
@@ -358,6 +358,8 @@ main (int argc, char **argv)
 	  add_tabstop (tabval);
 	  tabval = -1;
 	  break;
+	case_GETOPT_HELP_CHAR;
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 	default:
 	  if (tabval == -1)
 	    tabval = 0;
@@ -366,15 +368,6 @@ main (int argc, char **argv)
 	}
     }
 
-  if (show_version)
-    {
-      printf ("expand (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   add_tabstop (tabval);
 
   validate_tabstops (tab_list, first_free_tab);
