@@ -1,5 +1,5 @@
 /* GNU fmt -- simple text formatter.
-   Copyright (C) 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1994-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,18 +22,6 @@
 #include <sys/types.h>
 #include <getopt.h>
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
-#ifndef UINT_MAX
-# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
-#endif
-
-#ifndef INT_MAX
-# define INT_MAX ((int) (UINT_MAX >> 1))
-#endif
-
 /* Redefine.  Otherwise, systems (Unicos for one) with headers that define
    it to be a type get syntax errors for the variable declaration below.  */
 #define word unused_word_type
@@ -42,19 +30,24 @@
 #include "error.h"
 #include "xstrtol.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "fmt"
+
+#define AUTHORS "Ross Paterson"
+
 /* The following parameters represent the program's idea of what is
    "best".  Adjust to taste, subject to the caveats given.  */
 
 /* Default longest permitted line length (max_width).  */
-#define	WIDTH	75
+#define WIDTH	75
 
 /* Prefer lines to be LEEWAY % shorter than the maximum width, giving
    room for optimization.  */
-#define	LEEWAY	7
+#define LEEWAY	7
 
 /* The default secondary indent of tagged paragraph used for unindented
    one-line paragraphs not preceded by any multi-line paragraphs.  */
-#define	DEF_INDENT 3
+#define DEF_INDENT 3
 
 /* Costs and bonuses are expressed as the equivalent departure from the
    optimal line length, multiplied by 10.  e.g. assigning something a
@@ -64,62 +57,62 @@
 
 typedef long COST;
 
-#define	MAXCOST	(~(((unsigned long) 1) << (8 * sizeof (COST) -1)))
+#define MAXCOST	(~(((unsigned long) 1) << (8 * sizeof (COST) -1)))
 
-#define	SQR(n)		((n) * (n))
-#define	EQUIV(n)	SQR ((COST) (n))
+#define SQR(n)		((n) * (n))
+#define EQUIV(n)	SQR ((COST) (n))
 
 /* Cost of a filled line n chars longer or shorter than best_width.  */
-#define	SHORT_COST(n)	EQUIV ((n) * 10)
+#define SHORT_COST(n)	EQUIV ((n) * 10)
 
 /* Cost of the difference between adjacent filled lines.  */
-#define	RAGGED_COST(n)	(SHORT_COST (n) / 2)
+#define RAGGED_COST(n)	(SHORT_COST (n) / 2)
 
 /* Basic cost per line.  */
-#define	LINE_COST	EQUIV (70)
+#define LINE_COST	EQUIV (70)
 
 /* Cost of breaking a line after the first word of a sentence, where
    the length of the word is N.  */
-#define	WIDOW_COST(n)	(EQUIV (200) / ((n) + 2))
+#define WIDOW_COST(n)	(EQUIV (200) / ((n) + 2))
 
 /* Cost of breaking a line before the last word of a sentence, where
    the length of the word is N.  */
-#define	ORPHAN_COST(n)	(EQUIV (150) / ((n) + 2))
+#define ORPHAN_COST(n)	(EQUIV (150) / ((n) + 2))
 
 /* Bonus for breaking a line at the end of a sentence.  */
-#define	SENTENCE_BONUS	EQUIV (50)
+#define SENTENCE_BONUS	EQUIV (50)
 
 /* Cost of breaking a line after a period not marking end of a sentence.
    With the definition of sentence we are using (borrowed from emacs, see
    get_line()) such a break would then look like a sentence break.  Hence
    we assign a very high cost -- it should be avoided unless things are
    really bad.  */
-#define	NOBREAK_COST	EQUIV (600)
+#define NOBREAK_COST	EQUIV (600)
 
 /* Bonus for breaking a line before open parenthesis.  */
-#define	PAREN_BONUS	EQUIV (40)
+#define PAREN_BONUS	EQUIV (40)
 
 /* Bonus for breaking a line after other punctuation.  */
-#define	PUNCT_BONUS	EQUIV(40)
+#define PUNCT_BONUS	EQUIV(40)
 
 /* Credit for breaking a long paragraph one line later.  */
-#define	LINE_CREDIT	EQUIV(3)
+#define LINE_CREDIT	EQUIV(3)
 
 /* Size of paragraph buffer, in words and characters.  Longer paragraphs
    are handled neatly (cf. flush_paragraph()), so there's little to gain
    by making these larger.  */
-#define	MAXWORDS	1000
-#define	MAXCHARS	5000
+#define MAXWORDS	1000
+#define MAXCHARS	5000
 
 /* Extra ctype(3)-style macros.  */
 
-#define	isopen(c)	(strchr ("([`'\"", c) != NULL)
-#define	isclose(c)	(strchr (")]'\"", c) != NULL)
-#define	isperiod(c)	(strchr (".?!", c) != NULL)
+#define isopen(c)	(strchr ("([`'\"", c) != NULL)
+#define isclose(c)	(strchr (")]'\"", c) != NULL)
+#define isperiod(c)	(strchr (".?!", c) != NULL)
 
 /* Size of a tab stop, for expansion on input and re-introduction on
    output.  */
-#define	TABWIDTH	8
+#define TABWIDTH	8
 
 /* Miscellaneous definitions.  */
 
@@ -155,33 +148,27 @@ struct Word
 
 /* Forward declarations.  */
 
-static void set_prefix __P ((char *p));
-static void fmt __P ((FILE *f));
-static bool get_paragraph __P ((FILE *f));
-static int get_line __P ((FILE *f, int c));
-static int get_prefix __P ((FILE *f));
-static int get_space __P ((FILE *f, int c));
-static int copy_rest __P ((FILE *f, int c));
-static bool same_para __P ((int c));
-static void flush_paragraph __P ((void));
-static void fmt_paragraph __P ((void));
-static void check_punctuation __P ((WORD *w));
-static COST base_cost __P ((WORD *this));
-static COST line_cost __P ((WORD *next, int len));
-static void put_paragraph __P ((WORD *finish));
-static void put_line __P ((WORD *w, int indent));
-static void put_word __P ((WORD *w));
-static void put_space __P ((int space));
+static void set_prefix PARAMS ((char *p));
+static void fmt PARAMS ((FILE *f));
+static bool get_paragraph PARAMS ((FILE *f));
+static int get_line PARAMS ((FILE *f, int c));
+static int get_prefix PARAMS ((FILE *f));
+static int get_space PARAMS ((FILE *f, int c));
+static int copy_rest PARAMS ((FILE *f, int c));
+static bool same_para PARAMS ((int c));
+static void flush_paragraph PARAMS ((void));
+static void fmt_paragraph PARAMS ((void));
+static void check_punctuation PARAMS ((WORD *w));
+static COST base_cost PARAMS ((WORD *this));
+static COST line_cost PARAMS ((WORD *next, int len));
+static void put_paragraph PARAMS ((WORD *finish));
+static void put_line PARAMS ((WORD *w, int indent));
+static void put_word PARAMS ((WORD *w));
+static void put_space PARAMS ((int space));
 
 /* The name this program was run with.  */
 const char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help = 0;
-
-/* If nonzero, print the version on standard output and exit.  */
-static int show_version = 0;
-
 /* Option values.  */
 
 /* If TRUE, first 2 lines may have different indent (default FALSE).  */
@@ -274,7 +261,7 @@ static int next_prefix_indent;
    paragraphs chosen by fmt_paragraph().  */
 static int last_line_length;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -299,7 +286,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 \n\
 In -wNUMBER, the letter `w' may be omitted.\n"),
 	     stdout);
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -309,13 +296,13 @@ In -wNUMBER, the letter `w' may be omitted.\n"),
 static const struct option long_options[] =
 {
   {"crown-margin", no_argument, NULL, 'c'},
-  {"help", no_argument, &show_help, 1},
   {"prefix", required_argument, NULL, 'p'},
   {"split-only", no_argument, NULL, 's'},
   {"tagged-paragraph", no_argument, NULL, 't'},
   {"uniform-spacing", no_argument, NULL, 'u'},
-  {"version", no_argument, &show_version, 1},
   {"width", required_argument, NULL, 'w'},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {0, 0, 0, 0},
 };
 
@@ -323,7 +310,6 @@ int
 main (register int argc, register char **argv)
 {
   int optchar;
-  FILE *infile;
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
@@ -353,7 +339,7 @@ main (register int argc, register char **argv)
 
   while ((optchar = getopt_long (argc, argv, "0123456789cstuw:p:",
 				 long_options, NULL))
-	 != EOF)
+	 != -1)
     switch (optchar)
       {
       default:
@@ -393,36 +379,41 @@ main (register int argc, register char **argv)
 	set_prefix (optarg);
 	break;
 
+      case_GETOPT_HELP_CHAR;
+
+      case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
       }
 
-  if (show_version)
-    {
-      printf ("fmt (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   best_width = max_width * (2 * (100 - LEEWAY) + 1) / 200;
 
   if (optind == argc)
     fmt (stdin);
   else
-    for (; optind < argc; optind++)
-      if (strcmp (argv[optind], "-") == 0)
-	fmt (stdin);
-      else
+    {
+      for (; optind < argc; optind++)
 	{
-	  infile = fopen (argv[optind], "r");
-	  if (infile != NULL)
-	    {
-	      fmt (infile);
-	      fclose (infile);
-	    }
+	  char *file = argv[optind];
+	  if (STREQ (file, "-"))
+	    fmt (stdin);
 	  else
-	    error (0, errno, argv[optind]);
+	    {
+	      FILE *in_stream;
+	      in_stream = fopen (file, "r");
+	      if (in_stream != NULL)
+		{
+		  fmt (in_stream);
+		  if (fclose (in_stream) == EOF)
+		    error (EXIT_FAILURE, errno, "%s", file);
+		}
+	      else
+		error (0, errno, "%s", file);
+	    }
 	}
+    }
+
+  if (ferror (stdout) || fclose (stdout) == EOF)
+    error (EXIT_FAILURE, errno, _("write error"));
 
   exit (EXIT_SUCCESS);
 }
@@ -709,9 +700,9 @@ get_space (FILE *f, register int c)
 static void
 check_punctuation (register WORD *w)
 {
-  register const char *start, *finish;
+  const unsigned char *start, *finish;
 
-  start = w->text;
+  start = (unsigned char *) w->text;
   finish = start + (w->length - 1);
   w->paren = isopen (*start);
   w->punct = ISPUNCT (*finish);
