@@ -1,5 +1,5 @@
 /* fold -- wrap each input line to fit in specified width.
-   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,31 +19,18 @@
 
 #include <config.h>
 
-/* Get isblank from GNU libc.  */
-#define _GNU_SOURCE
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
-#ifndef UINT_MAX
-# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
-#endif
-
-#ifndef INT_MAX
-# define INT_MAX ((int) (UINT_MAX >> 1))
-#endif
-
 #include "system.h"
-#include "xstrtol.h"
 #include "error.h"
+#include "xstrtol.h"
 
-char *xrealloc ();
-char *xmalloc ();
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "fold"
+
+#define AUTHORS "David MacKenzie"
 
 /* The name this program was run with. */
 char *program_name;
@@ -57,23 +44,17 @@ static int count_bytes;
 /* If nonzero, at least one of the files we read was standard input. */
 static int have_read_stdin;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
   {"bytes", no_argument, NULL, 'b'},
   {"spaces", no_argument, NULL, 's'},
   {"width", required_argument, NULL, 'w'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -92,8 +73,10 @@ standard output.\n\
   -b, --bytes         count bytes rather than columns\n\
   -s, --spaces        break at spaces\n\
   -w, --width=WIDTH   use WIDTH columns instead of 80\n\
+      --help          display this help and exit\n\
+      --version       output version information and exit\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -138,7 +121,7 @@ fold_file (char *filename, int width)
   static char *line_out = NULL;
   static int allocated_out = 0;
 
-  if (!strcmp (filename, "-"))
+  if (STREQ (filename, "-"))
     {
       istream = stdin;
       have_read_stdin = 1;
@@ -227,11 +210,11 @@ fold_file (char *filename, int width)
   if (ferror (istream))
     {
       error (0, errno, "%s", filename);
-      if (strcmp (filename, "-"))
+      if (!STREQ (filename, "-"))
 	fclose (istream);
       return 1;
     }
-  if (strcmp (filename, "-") && fclose (istream) == EOF)
+  if (!STREQ (filename, "-") && fclose (istream) == EOF)
     {
       error (0, errno, "%s", filename);
       return 1;
@@ -276,8 +259,7 @@ main (int argc, char **argv)
 	}
     }
 
-  while ((optc = getopt_long (argc, argv, "bsw:", longopts, (int *) 0))
-	 != EOF)
+  while ((optc = getopt_long (argc, argv, "bsw:", longopts, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -303,20 +285,15 @@ main (int argc, char **argv)
 	  }
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("fold (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (argc == optind)
     errs |= fold_file ("-", width);
   else
