@@ -1,5 +1,5 @@
 /* head -- output first part of file(s)
-   Copyright (C) 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +31,13 @@
 #include <sys/types.h>
 #include "system.h"
 #include "error.h"
+#include "xstrtol.h"
+#include "safe-read.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "head"
+
+#define AUTHORS "David MacKenzie"
 
 /* Number of lines/chars/blocks to head. */
 #define DEFAULT_NUMBER 10
@@ -38,10 +45,6 @@
 /* Size of atomic reads. */
 #define BUFSIZE (512 * 8)
 
-/* Number of bytes per item we are printing.
-   If 0, head in lines. */
-static int unit_size;
-
 /* If nonzero, print filename headers. */
 static int print_headers;
 
@@ -51,20 +54,12 @@ enum header_mode
   multiple_files, always, never
 };
 
-int safe_read ();
-
 /* The name this program was run with. */
 char *program_name;
 
 /* Have we ever read standard input?  */
 static int have_read_stdin;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
   {"bytes", required_argument, NULL, 'c'},
@@ -72,12 +67,12 @@ static struct option const long_options[] =
   {"quiet", no_argument, NULL, 'q'},
   {"silent", no_argument, NULL, 'q'},
   {"verbose", no_argument, NULL, 'v'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -105,49 +100,11 @@ SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
 If -VALUE is used as first OPTION, read -c VALUE when one of\n\
 multipliers bkm follows concatenated, else read -n VALUE.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
-/* Convert STR, a string of ASCII digits, into an unsigned integer.
-   Return -1 if STR does not represent a valid unsigned integer. */
-
-static long
-atou (const char *str)
-{
-  int value;
-
-  for (value = 0; ISDIGIT (*str); ++str)
-    value = value * 10 + *str - '0';
-  return *str ? -1 : value;
-}
-
-static void
-parse_unit (char *str)
-{
-  int arglen = strlen (str);
-
-  if (arglen == 0)
-    return;
-
-  switch (str[arglen - 1])
-    {
-    case 'b':
-      unit_size = 512;
-      str[arglen - 1] = '\0';
-      break;
-    case 'k':
-      unit_size = 1024;
-      str[arglen - 1] = '\0';
-      break;
-    case 'm':
-      unit_size = 1048576;
-      str[arglen - 1] = '\0';
-      break;
-    }
-}
-
 static void
 write_header (const char *filename)
 {
@@ -158,11 +115,14 @@ write_header (const char *filename)
 }
 
 static int
-head_bytes (const char *filename, int fd, long int bytes_to_write)
+head_bytes (const char *filename, int fd, uintmax_t bytes_to_write)
 {
   char buffer[BUFSIZE];
   int bytes_read;
 
+  /* Need BINARY I/O for the byte counts to be accurate.  */
+  SET_BINARY2 (fd, fileno (stdout));
+
   while (bytes_to_write)
     {
       bytes_read = safe_read (fd, buffer, BUFSIZE);
@@ -183,15 +143,18 @@ head_bytes (const char *filename, int fd, long int bytes_to_write)
 }
 
 static int
-head_lines (const char *filename, int fd, long int lines_to_write)
+head_lines (const char *filename, int fd, uintmax_t lines_to_write)
 {
   char buffer[BUFSIZE];
-  int bytes_read;
-  int bytes_to_write;
+
+  /* Need BINARY I/O for the byte counts to be accurate.  */
+  SET_BINARY2 (fd, fileno (stdout));
 
   while (lines_to_write)
     {
-      bytes_read = safe_read (fd, buffer, BUFSIZE);
+      int bytes_read = safe_read (fd, buffer, BUFSIZE);
+      int bytes_to_write = 0;
+
       if (bytes_read < 0)
 	{
 	  error (0, errno, "%s", filename);
@@ -199,7 +162,6 @@ head_lines (const char *filename, int fd, long int lines_to_write)
 	}
       if (bytes_read == 0)
 	break;
-      bytes_to_write = 0;
       while (bytes_to_write < bytes_read)
 	if (buffer[bytes_to_write++] == '\n' && --lines_to_write == 0)
 	  break;
@@ -210,26 +172,26 @@ head_lines (const char *filename, int fd, long int lines_to_write)
 }
 
 static int
-head (const char *filename, int fd, long int number)
+head (const char *filename, int fd, uintmax_t n_units, int count_lines)
 {
-  if (unit_size)
-    return head_bytes (filename, fd, number);
+  if (count_lines)
+    return head_lines (filename, fd, n_units);
   else
-    return head_lines (filename, fd, number);
+    return head_bytes (filename, fd, n_units);
 }
 
 static int
-head_file (const char *filename, long int number)
+head_file (const char *filename, uintmax_t n_units, int count_lines)
 {
   int fd;
 
-  if (!strcmp (filename, "-"))
+  if (STREQ (filename, "-"))
     {
       have_read_stdin = 1;
       filename = _("standard input");
       if (print_headers)
 	write_header (filename);
-      return head (filename, 0, number);
+      return head (filename, 0, n_units, count_lines);
     }
   else
     {
@@ -240,7 +202,7 @@ head_file (const char *filename, long int number)
 
 	  if (print_headers)
 	    write_header (filename);
-	  errors = head (filename, fd, number);
+	  errors = head (filename, fd, n_units, count_lines);
 	  if (close (fd) == 0)
 	    return errors;
 	}
@@ -249,13 +211,52 @@ head_file (const char *filename, long int number)
     }
 }
 
+/* Convert a string of digits, N_STRING, with a single, optional suffix
+   character (b, k, or m) to an integral value.  Upon successful conversion,
+   return that value.  If it cannot be converted, give a diagnostic and exit.
+   COUNT_LINES indicates whether N_STRING is a number of bytes or a number
+   of lines.  It is used solely to give a more specific diagnostic.  */
+
+static uintmax_t
+string_to_integer (int count_lines, const char *n_string)
+{
+  strtol_error s_err;
+  uintmax_t n;
+
+  s_err = xstrtoumax (n_string, NULL, 0, &n, "bkm");
+
+  if (s_err == LONGINT_INVALID)
+    {
+      error (EXIT_FAILURE, 0, "%s: %s", n_string,
+	     (count_lines
+	      ? _("invalid number of lines")
+	      : _("invalid number of bytes")));
+    }
+
+  if (s_err != LONGINT_OK)
+    {
+      error (EXIT_FAILURE, 0,
+	     _("%s: %s is so large that it is not representable"), n_string,
+	     count_lines ? _("number of lines") : _("number of bytes"));
+    }
+
+  return n;
+}
+
 int
 main (int argc, char **argv)
 {
   enum header_mode header_mode = multiple_files;
   int exit_status = 0;
-  long number = -1;		/* Number of items to print (-1 if undef.). */
-  int c;			/* Option character. */
+  char *n_string;
+  int c;
+
+  /* Number of items to print. */
+  uintmax_t n_units = DEFAULT_NUMBER;
+
+  /* If nonzero, interpret the numeric argument as the number of lines.
+     Otherwise, interpret it as the number of bytes.  */
+  int count_lines = 1;
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
@@ -263,38 +264,45 @@ main (int argc, char **argv)
   textdomain (PACKAGE);
 
   have_read_stdin = 0;
-  unit_size = 0;
+
   print_headers = 0;
 
   if (argc > 1 && argv[1][0] == '-' && ISDIGIT (argv[1][1]))
     {
+      char *end_n_string;
+      char multiplier_char = 0;
+
+      n_string = &argv[1][1];
+
       /* Old option syntax; a dash, one or more digits, and one or
 	 more option letters.  Move past the number. */
-      for (number = 0, ++argv[1]; ISDIGIT (*argv[1]); ++argv[1])
-	number = number * 10 + *argv[1] - '0';
+      for (++argv[1]; ISDIGIT (*argv[1]); ++argv[1])
+	{
+	  /* empty */
+	}
+
+      /* Pointer to the byte after the last digit.  */
+      end_n_string = argv[1];
+
       /* Parse any appended option letters. */
       while (*argv[1])
 	{
 	  switch (*argv[1])
 	    {
-	    case 'b':
-	      unit_size = 512;
-	      break;
-
 	    case 'c':
-	      unit_size = 1;
+	      count_lines = 0;
+	      multiplier_char = 0;
 	      break;
 
+	    case 'b':
 	    case 'k':
-	      unit_size = 1024;
+	    case 'm':
+	      count_lines = 0;
+	      multiplier_char = *argv[1];
 	      break;
 
 	    case 'l':
-	      unit_size = 0;
-	      break;
-
-	    case 'm':
-	      unit_size = 1048576;
+	      count_lines = 1;
 	      break;
 
 	    case 'q':
@@ -311,14 +319,25 @@ main (int argc, char **argv)
 	    }
 	  ++argv[1];
 	}
+
+      /* Append the multiplier character (if any) onto the end of
+	 the digit string.  Then add NUL byte if necessary.  */
+      *end_n_string = multiplier_char;
+      if (multiplier_char)
+	*(++end_n_string) = 0;
+
+      n_units = string_to_integer (count_lines, n_string);
+
       /* Make the options we just parsed invisible to getopt. */
       argv[1] = argv[0];
       argv++;
       argc--;
+
+      /* FIXME: allow POSIX options if there were obsolescent ones?  */
+
     }
 
-  while ((c = getopt_long (argc, argv, "c:n:qv", long_options, (int *) 0))
-	 != EOF)
+  while ((c = getopt_long (argc, argv, "c:n:qv", long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -326,16 +345,13 @@ main (int argc, char **argv)
 	  break;
 
 	case 'c':
-	  unit_size = 1;
-	  parse_unit (optarg);
-	  goto getnum;
+	  count_lines = 0;
+	  n_units = string_to_integer (count_lines, optarg);
+	  break;
+
 	case 'n':
-	  unit_size = 0;
-	getnum:
-	  /* FIXME: use xstrtoul instead.  */
-	  number = atou (optarg);
-	  if (number == -1)
-	    error (EXIT_FAILURE, 0, _("invalid number `%s'"), optarg);
+	  count_lines = 1;
+	  n_units = string_to_integer (count_lines, optarg);
 	  break;
 
 	case 'q':
@@ -346,35 +362,24 @@ main (int argc, char **argv)
 	  header_mode = always;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("head (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
-  if (number == -1)
-    number = DEFAULT_NUMBER;
-
-  if (unit_size > 1)
-    number *= unit_size;
-
   if (header_mode == always
       || (header_mode == multiple_files && optind < argc - 1))
     print_headers = 1;
 
   if (optind == argc)
-    exit_status |= head_file ("-", number);
+    exit_status |= head_file ("-", n_units, count_lines);
 
   for (; optind < argc; ++optind)
-    exit_status |= head_file (argv[optind], number);
+    exit_status |= head_file (argv[optind], n_units, count_lines);
 
   if (have_read_stdin && close (0) < 0)
     error (EXIT_FAILURE, errno, "-");
