@@ -1,5 +1,5 @@
 /* join - join lines of two files on a common field
-   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,58 +19,26 @@
 
 #include <config.h>
 
-#ifdef __GNUC__
-#define alloca __builtin_alloca
-#else /* not __GNUC__ */
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#else /* not HAVE_ALLOCA_H */
-#ifdef _AIX
- #pragma alloca
-#else /* not _AIX */
-char *alloca ();
-#endif /* not _AIX */
-#endif /* not HAVE_ALLOCA_H */
-#endif /* not __GNUC__ */
-
-/* Get isblank from GNU libc.  */
-#define _GNU_SOURCE
-
 #include <stdio.h>
-#define NDEBUG
 #include <assert.h>
 #include <sys/types.h>
 #include <getopt.h>
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
-#ifndef UINT_MAX
-# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
-#endif
-
-#ifndef INT_MAX
-# define INT_MAX ((int) (UINT_MAX >> 1))
-#endif
-
-#if _LIBC || STDC_HEADERS
-# define TOLOWER(c) tolower (c)
-#else
-# define TOLOWER(c) (ISUPPER (c) ? tolower (c) : (c))
-#endif
-
 #include "system.h"
-#include "long-options.h"
-#include "xstrtol.h"
 #include "error.h"
+#include "hard-locale.h"
+#include "linebuffer.h"
 #include "memcasecmp.h"
+#include "memcoll.h"
+#include "xstrtol.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "join"
+
+#define AUTHORS "Mike Haertel"
 
 #define join system_join
 
-char *xmalloc ();
-char *xrealloc ();
-
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
 #undef max
@@ -94,15 +62,14 @@ struct outlist
 /* A field of a line.  */
 struct field
   {
-    const char *beg;		/* First character in field.  */
+    const unsigned char *beg;	/* First character in field.  */
     size_t len;			/* The length of the field.  */
   };
 
-/* A line read from an input file.  Newlines are not stored.  */
+/* A line read from an input file.  */
 struct line
   {
-    char *beg;			/* First character in line.  */
-    char *lim;			/* Character after last character in line.  */
+    struct linebuffer buf;	/* The line itself.  */
     int nfields;		/* Number of elements in `fields'.  */
     int nfields_allocated;	/* Number of elements in `fields'.  */
     struct field *fields;
@@ -120,6 +87,11 @@ struct seq
 /* The name this program was run with.  */
 char *program_name;
 
+#ifdef ENABLE_NLS
+/* Nonzero if the LC_COLLATE locale is hard.  */
+static int hard_LC_COLLATE;
+#endif
+
 /* If nonzero, print unpairable lines in file 1 or 2.  */
 static int print_unpairables_1, print_unpairables_2;
 
@@ -151,6 +123,8 @@ static struct option const longopts[] =
   {"j", required_argument, NULL, 'j'},
   {"j1", required_argument, NULL, '1'},
   {"j2", required_argument, NULL, '2'},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
@@ -160,7 +134,7 @@ static struct line uni_blank;
 /* If nonzero, ignore case when comparing join fields.  */
 static int ignore_case;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -198,13 +172,13 @@ each being `SIDE.FIELD' or `0'.  Default FORMAT outputs the join field,\n\
 the remaining fields from FILE1, the remaining fields from FILE2, all\n\
 separated by CHAR.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static void
-ADD_FIELD (struct line *line, const char *field, size_t len)
+ADD_FIELD (struct line *line, const unsigned char *field, size_t len)
 {
   if (line->nfields >= line->nfields_allocated)
     {
@@ -224,15 +198,17 @@ static void
 xfields (struct line *line)
 {
   int i;
-  register char *ptr, *lim;
+  unsigned char *ptr0 = (unsigned char *) line->buf.buffer;
+  unsigned char *ptr;
+  unsigned char *lim;
 
-  ptr = line->beg;
-  lim = line->lim;
+  ptr = ptr0;
+  lim = ptr0 + line->buf.length - 1;
 
   if (!tab)
     {
       /* Skip leading blanks before the first field.  */
-      while (ptr < lim && ISSPACE (*ptr))
+      while (ptr < lim && ISBLANK (*ptr))
 	++ptr;
     }
 
@@ -240,7 +216,7 @@ xfields (struct line *line)
     {
       if (tab)
 	{
-	  char *beg;
+	  unsigned char *beg;
 
 	  beg = ptr;
 	  while (ptr < lim && *ptr != tab)
@@ -251,18 +227,18 @@ xfields (struct line *line)
 	}
       else
 	{
-	  char *beg;
+	  unsigned char *beg;
 
 	  beg = ptr;
-	  while (ptr < lim && !ISSPACE (*ptr))
+	  while (ptr < lim && !ISBLANK (*ptr))
 	    ++ptr;
 	  ADD_FIELD (line, beg, ptr - beg);
-	  while (ptr < lim && ISSPACE (*ptr))
+	  while (ptr < lim && ISBLANK (*ptr))
 	    ++ptr;
 	}
     }
 
-  if (ptr > line->beg && ((tab && ISSPACE (ptr[-1])) || ptr[-1] == tab))
+  if (ptr != ptr0 && ((!tab && ISBLANK (ptr[-1])) || ptr[-1] == tab))
     {
       /* Add one more (empty) field because the last character of the
 	 line was a delimiter.  */
@@ -276,33 +252,15 @@ xfields (struct line *line)
 static int
 get_line (FILE *fp, struct line *line)
 {
-  static int linesize = 80;
-  int c, i;
-  char *ptr;
+  initbuffer (&line->buf);
 
-  if (feof (fp))
-    return 0;
-
-  ptr = xmalloc (linesize);
-
-  for (i = 0; (c = getc (fp)) != EOF && c != '\n'; ++i)
+  if (! readline (&line->buf, fp))
     {
-      if (i == linesize)
-	{
-	  linesize *= 2;
-	  ptr = xrealloc (ptr, linesize);
-	}
-      ptr[i] = c;
-    }
-
-  if (c == EOF && i == 0)
-    {
-      free (ptr);
+      free (line->buf.buffer);
+      line->buf.buffer = NULL;
       return 0;
     }
 
-  line->beg = ptr;
-  line->lim = line->beg + i;
   line->nfields_allocated = 0;
   line->nfields = 0;
   line->fields = NULL;
@@ -314,8 +272,8 @@ static void
 freeline (struct line *line)
 {
   free ((char *) line->fields);
-  free (line->beg);
-  line->beg = NULL;
+  free (line->buf.buffer);
+  line->buf.buffer = NULL;
 }
 
 static void
@@ -351,7 +309,7 @@ delseq (struct seq *seq)
 {
   int i;
   for (i = 0; i < seq->count; i++)
-    if (seq->lines[i].beg)
+    if (seq->lines[i].buf.buffer)
       freeline (&seq->lines[i]);
   free ((char *) seq->lines);
 }
@@ -362,7 +320,9 @@ delseq (struct seq *seq)
 static int
 keycmp (struct line *line1, struct line *line2)
 {
-  const char *beg1, *beg2;	/* Start of field to compare in each file.  */
+  /* Start of field to compare in each file.  */
+  const unsigned char *beg1, *beg2;
+
   int len1, len2;		/* Length of fields to compare.  */
   int diff;
 
@@ -397,9 +357,19 @@ keycmp (struct line *line1, struct line *line2)
      avoid portability hassles of getting a non-conflicting declaration
      of memcmp.  */
   if (ignore_case)
-    diff = memcasecmp (beg1, beg2, min (len1, len2));
+    {
+      /* FIXME: ignore_case does not work with NLS (in particular,
+         with multibyte chars).  */
+      diff = memcasecmp (beg1, beg2, min (len1, len2));
+    }
   else
-    diff = memcmp (beg1, beg2, min (len1, len2));
+    {
+#ifdef ENABLE_NLS
+      if (hard_LC_COLLATE)
+	return memcoll ((char *) beg1, len1, (char *) beg2, len2);
+#endif
+      diff = memcmp (beg1, beg2, min (len1, len2));
+    }
 
   if (diff)
     return diff;
@@ -738,17 +708,20 @@ void
 make_blank (struct line *blank, int count)
 {
   int i;
+  char *buffer;
+  struct field *fields;
   blank->nfields = count;
-  blank->beg = xmalloc (blank->nfields + 1);
-  blank->fields = (struct field *) xmalloc (sizeof (struct field) * count);
-  for (i = 0; i < blank->nfields; i++)
+  blank->buf.size = blank->buf.length = count + 1;
+  blank->buf.buffer = buffer = xmalloc (blank->buf.size);
+  blank->fields = fields =
+    (struct field *) xmalloc (sizeof (struct field) * count);
+  for (i = 0; i < count; i++)
     {
-      blank->beg[i] = '\t';
-      blank->fields[i].beg = &blank->beg[i];
-      blank->fields[i].len = 0;
+      buffer[i] = '\t';
+      fields[i].beg = &buffer[i];
+      fields[i].len = 0;
     }
-  blank->beg[i] = '\0';
-  blank->lim = &blank->beg[i];
+  buffer[i] = '\n';
 }
 
 int
@@ -763,17 +736,19 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+#ifdef ENABLE_NLS
+  hard_LC_COLLATE = hard_locale (LC_COLLATE);
+#endif
+
   /* Initialize this before parsing options.  In parsing options,
      it may be increased.  */
   uni_blank.nfields = 1;
 
-  parse_long_options (argc, argv, "join", GNU_PACKAGE, VERSION, usage);
-
   nfiles = 0;
   print_pairables = 1;
 
   while ((optc = getopt_long_only (argc, argv, "-a:e:i1:2:o:t:v:", longopts,
-				   (int *) 0)) != EOF)
+				   NULL)) != -1)
     {
       long int val;
 
@@ -856,7 +831,11 @@ main (int argc, char **argv)
 	  names[nfiles++] = optarg;
 	  break;
 
-	case '?':
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
+	default:
 	  usage (1);
 	}
       prev_optc = optc;
@@ -872,10 +851,10 @@ main (int argc, char **argv)
       usage (1);
     }
 
-  fp1 = strcmp (names[0], "-") ? fopen (names[0], "r") : stdin;
+  fp1 = STREQ (names[0], "-") ? stdin : fopen (names[0], "r");
   if (!fp1)
     error (EXIT_FAILURE, errno, "%s", names[0]);
-  fp2 = strcmp (names[1], "-") ? fopen (names[1], "r") : stdin;
+  fp2 = STREQ (names[1], "-") ? stdin : fopen (names[1], "r");
   if (!fp2)
     error (EXIT_FAILURE, errno, "%s", names[1]);
   if (fp1 == fp2)
