@@ -1,6 +1,6 @@
 /* Compute MD5 checksum of files or strings according to the definition
    of MD5 in RFC 1321 from April 1992.
-   Copyright (C) 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,37 +26,37 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-#include "long-options.h"
 #include "md5.h"
 #include "getline.h"
 #include "system.h"
 #include "error.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "md5sum"
+
+#define AUTHORS "Ulrich Drepper"
+
 /* Most systems do not distinguish between external and internal
    text representations.  */
-#if UNIX || __UNIX__ || unix || __unix__ || _POSIX_VERSION
-# define OPENOPTS(BINARY) "r"
+/* FIXME: This begs for an autoconf test.  */
+#if O_BINARY
+# define OPENOPTS(BINARY) ((BINARY) != 0 ? TEXT1TO1 : TEXTCNVT)
+# define TEXT1TO1 "rb"
+# define TEXTCNVT "r"
 #else
-# ifdef MSDOS
-#  define TEXT1TO1 "rb"
-#  define TEXTCNVT "r"
+# if defined VMS
+#  define OPENOPTS(BINARY) ((BINARY) != 0 ? TEXT1TO1 : TEXTCNVT)
+#  define TEXT1TO1 "rb", "ctx=stm"
+#  define TEXTCNVT "r", "ctx=stm"
 # else
-#  if defined VMS
-#   define TEXT1TO1 "rb", "ctx=stm"
-#   define TEXTCNVT "r", "ctx=stm"
+#  if UNIX || __UNIX__ || unix || __unix__ || _POSIX_VERSION
+#   define OPENOPTS(BINARY) "r"
 #  else
     /* The following line is intended to evoke an error.
        Using #error is not portable enough.  */
     "Cannot determine system type."
 #  endif
 # endif
-# define OPENOPTS(BINARY) ((BINARY) != 0 ? TEXT1TO1 : TEXTCNVT)
-#endif
-
-#if _LIBC || STDC_HEADERS
-# define TOLOWER(c) tolower (c)
-#else
-# define TOLOWER(c) (ISUPPER (c) ? tolower (c) : (c))
 #endif
 
 /* The minimum length of a valid digest line in a file produced
@@ -88,12 +88,12 @@ static const struct option long_options[] =
   { "string", required_argument, 0, 1 },
   { "text", no_argument, 0, 't' },
   { "warn", no_argument, 0, 'w' },
+  { GETOPT_HELP_OPTION_DECL },
+  { GETOPT_VERSION_OPTION_DECL },
   { NULL, 0, NULL, 0 }
 };
 
-char *xmalloc ();
-
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -107,7 +107,7 @@ Usage: %s [OPTION] [FILE]...\n\
 Print or check MD5 checksums.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
-  -b, --binary            read files in binary mode\n\
+  -b, --binary            read files in binary mode (default on DOS/Windows)\n\
   -c, --check             check MD5 sums against given list\n\
   -t, --text              read files in text mode (default)\n\
 \n\
@@ -122,18 +122,18 @@ The sums are computed as described in RFC 1321.  When checking, the input\n\
 should be a former output of this program.  The default mode is to print\n\
 a line with checksum, a character indicating type (`*' for binary, ` ' for\n\
 text), and name for each FILE.\n"),
-	      program_name, program_name, program_name);
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+	      program_name, program_name);
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
 
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static int
-split_3 (char *s, size_t s_len, char **u, int *binary, char **w)
+split_3 (char *s, size_t s_len, unsigned char **u, int *binary, char **w)
 {
   size_t i;
-  int filename_has_newline = 0;
+  int escaped_filename = 0;
 
 #define ISWHITE(c) ((c) == ' ' || (c) == '\t')
 
@@ -151,9 +151,9 @@ split_3 (char *s, size_t s_len, char **u, int *binary, char **w)
   if (s[i] == '\\')
     {
       ++i;
-      filename_has_newline = 1;
+      escaped_filename = 1;
     }
-  *u = &s[i];
+  *u = (unsigned char *) &s[i];
 
   /* The first field has to be the 32-character hexadecimal
      representation of the message digest.  If it is not followed
@@ -172,7 +172,7 @@ split_3 (char *s, size_t s_len, char **u, int *binary, char **w)
      significant -- that includes leading and trailing white space.  */
   *w = &s[i];
 
-  if (filename_has_newline)
+  if (escaped_filename)
     {
       /* Translate each `\n' string in the file name to a NEWLINE,
 	 and each `\\' string to a backslash.  */
@@ -220,7 +220,7 @@ split_3 (char *s, size_t s_len, char **u, int *binary, char **w)
 }
 
 static int
-hex_digits (const char *s)
+hex_digits (unsigned char const *s)
 {
   while (*s)
     {
@@ -241,10 +241,16 @@ md5_file (const char *filename, int binary, unsigned char *md5_result)
   FILE *fp;
   int err;
 
-  if (strcmp (filename, "-") == 0)
+  if (STREQ (filename, "-"))
     {
       have_read_stdin = 1;
       fp = stdin;
+#if O_BINARY
+      /* If we need binary reads from a pipe or redirected stdin, we need
+	 to switch it to BINARY mode here, since stdin is already open.  */
+      if (binary)
+	SET_BINARY (fileno (stdin));
+#endif
     }
   else
     {
@@ -290,7 +296,7 @@ md5_check (const char *checkfile_name)
   char *line;
   size_t line_chars_allocated;
 
-  if (strcmp (checkfile_name, "-") == 0)
+  if (STREQ (checkfile_name, "-"))
     {
       have_read_stdin = 1;
       checkfile_name = _("standard input");
@@ -313,7 +319,7 @@ md5_check (const char *checkfile_name)
     {
       char *filename;
       int binary;
-      char *md5num;
+      unsigned char *md5num;
       int err;
       int line_length;
 
@@ -448,12 +454,17 @@ main (int argc, char **argv)
   int opt;
   char **string = NULL;
   size_t n_strings = 0;
-  size_t i;
   size_t err = 0;
   int file_type_specified = 0;
 
+#if O_BINARY
+  /* Binary is default on MSDOS, so the actual file contents
+     are used in computation.  */
+  int binary = 1;
+#else
   /* Text is default of the Plumb/Lankester format.  */
   int binary = 0;
+#endif
 
   /* Setting values of global variables.  */
   program_name = argv[0];
@@ -461,10 +472,7 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "md5sum", GNU_PACKAGE, VERSION, usage);
-
-  while ((opt = getopt_long (argc, argv, "bctw", long_options, NULL))
-	 != EOF)
+  while ((opt = getopt_long (argc, argv, "bctw", long_options, NULL)) != -1)
     switch (opt)
       {
       case 0:			/* long option */
@@ -498,6 +506,8 @@ main (int argc, char **argv)
 	status_only = 0;
 	warn = 1;
 	break;
+      case_GETOPT_HELP_CHAR;
+      case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
       default:
 	usage (EXIT_FAILURE);
       }
@@ -532,6 +542,8 @@ verifying checksums"));
 
   if (n_strings > 0)
     {
+      size_t i;
+
       if (optind < argc)
 	{
 	  error (0, 0, _("no files may be specified when using --string"));
@@ -576,8 +588,8 @@ verifying checksums"));
 	      size_t i;
 
 	      /* Output a leading backslash if the file name contains
-		 a newline.  */
-	      if (strchr (file, '\n'))
+		 a newline or backslash.  */
+	      if (strchr (file, '\n') || strchr (file, '\\'))
 		putchar ('\\');
 
 	      for (i = 0; i < 16; ++i)
