@@ -1,5 +1,5 @@
 /* nl -- number lines of files
-   Copyright (C) 89, 92, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 89, 92, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,32 +24,22 @@
 #include <sys/types.h>
 #include <getopt.h>
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
-#if WITH_REGEX
-# include <regex.h>
-#else
-# include <rx.h>
-#endif
-
-#ifndef UINT_MAX
-# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
-#endif
-
-#ifndef INT_MAX
-# define INT_MAX ((int) (UINT_MAX >> 1))
-#endif
-
-#include "linebuffer.h"
 #include "system.h"
+
+#include <regex.h>
+
 #include "error.h"
+#include "linebuffer.h"
 #include "xstrtol.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "nl"
+
+#define AUTHORS "Scott Bartram and David MacKenzie"
+
 #ifndef TRUE
-#define TRUE   1
-#define FALSE  0
+# define TRUE   1
+# define FALSE  0
 #endif
 
 /* Line-number formats. */
@@ -70,9 +60,6 @@ enum section
   Header, Body, Footer, Text
 };
 
-char *xmalloc ();
-char *xrealloc ();
-
 /* The name this program was run with. */
 char *program_name;
 
@@ -110,19 +97,19 @@ static char *section_del = DEFAULT_SECTION_DELIMITERS;
 static char *header_del = NULL;
 
 /* Header section delimiter length.  */
-static int header_del_len;
+static size_t header_del_len;
 
 /* Body delimiter string.  */
 static char *body_del = NULL;
 
 /* Body section delimiter length.  */
-static int body_del_len;
+static size_t body_del_len;
 
 /* Footer delimiter string.  */
 static char *footer_del = NULL;
 
 /* Footer section delimiter length.  */
-static int footer_del_len;
+static size_t footer_del_len;
 
 /* Input buffer.  */
 static struct linebuffer line_buf;
@@ -157,12 +144,6 @@ static int line_no;
 /* Nonzero if we have ever read standard input. */
 static int have_read_stdin;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
   {"header-numbering", required_argument, NULL, 'h'},
@@ -176,14 +157,14 @@ static struct option const longopts[] =
   {"number-width", required_argument, NULL, 'w'},
   {"number-format", required_argument, NULL, 'n'},
   {"section-delimiter", required_argument, NULL, 'd'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
 /* Print a usage message and quit. */
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -229,7 +210,7 @@ FORMAT is one of:\n\
   rz   right justified, leading zeros\n\
 \n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -344,7 +325,7 @@ proc_text (void)
     case 'a':
       if (blank_join > 1)
 	{
-	  if (line_buf.length || ++blank_lines == blank_join)
+	  if (1 < line_buf.length || ++blank_lines == blank_join)
 	    {
 	      print_lineno ();
 	      blank_lines = 0;
@@ -356,7 +337,7 @@ proc_text (void)
 	print_lineno ();
       break;
     case 't':
-      if (line_buf.length)
+      if (1 < line_buf.length)
 	print_lineno ();
       else
 	printf (print_no_line_fmt);
@@ -365,15 +346,14 @@ proc_text (void)
       printf (print_no_line_fmt);
       break;
     case 'p':
-      if (re_search (current_regex, line_buf.buffer, line_buf.length,
-		     0, line_buf.length, (struct re_registers *) 0) < 0)
+      if (re_search (current_regex, line_buf.buffer, line_buf.length - 1,
+		     0, line_buf.length - 1, (struct re_registers *) 0) < 0)
 	printf (print_no_line_fmt);
       else
 	print_lineno ();
       break;
     }
   fwrite (line_buf.buffer, sizeof (char), line_buf.length, stdout);
-  putchar ('\n');
 }
 
 /* Return the type of line in `line_buf'. */
@@ -381,15 +361,17 @@ proc_text (void)
 static enum section
 check_section (void)
 {
-  if (line_buf.length < 2 || memcmp (line_buf.buffer, section_del, 2))
+  size_t len = line_buf.length - 1;
+  
+  if (len < 2 || memcmp (line_buf.buffer, section_del, 2))
     return Text;
-  if (line_buf.length == header_del_len
+  if (len == header_del_len
       && !memcmp (line_buf.buffer, header_del, header_del_len))
     return Header;
-  if (line_buf.length == body_del_len
+  if (len == body_del_len
       && !memcmp (line_buf.buffer, body_del, body_del_len))
     return Body;
-  if (line_buf.length == footer_del_len
+  if (len == footer_del_len
       && !memcmp (line_buf.buffer, footer_del, footer_del_len))
     return Footer;
   return Text;
@@ -428,7 +410,7 @@ nl_file (const char *file)
 {
   FILE *stream;
 
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     {
       have_read_stdin = 1;
       stream = stdin;
@@ -450,7 +432,7 @@ nl_file (const char *file)
       error (0, errno, "%s", file);
       return 1;
     }
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     clearerr (stream);		/* Also clear EOF. */
   else if (fclose (stream) == EOF)
     {
@@ -464,6 +446,7 @@ int
 main (int argc, char **argv)
 {
   int c, exit_status = 0;
+  size_t len;
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
@@ -473,7 +456,7 @@ main (int argc, char **argv)
   have_read_stdin = 0;
 
   while ((c = getopt_long (argc, argv, "h:b:f:v:i:pl:s:w:n:d:", longopts,
-			   (int *) 0)) != EOF)
+			   NULL)) != -1)
     {
       switch (c)
 	{
@@ -571,33 +554,26 @@ main (int argc, char **argv)
 	case 'd':
 	  section_del = optarg;
 	  break;
+	case_GETOPT_HELP_CHAR;
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 	default:
 	  usage (2);
 	  break;
 	}
     }
 
-  if (show_version)
-    {
-      printf ("nl (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   /* Initialize the section delimiters.  */
-  c = strlen (section_del);
+  len = strlen (section_del);
 
-  header_del_len = c * 3;
+  header_del_len = len * 3;
   header_del = xmalloc (header_del_len + 1);
   strcat (strcat (strcpy (header_del, section_del), section_del), section_del);
 
-  body_del_len = c * 2;
+  body_del_len = len * 2;
   body_del = xmalloc (body_del_len + 1);
   strcat (strcpy (body_del, section_del), section_del);
 
-  footer_del_len = c;
+  footer_del_len = len;
   footer_del = xmalloc (footer_del_len + 1);
   strcpy (footer_del, section_del);
 
@@ -605,10 +581,10 @@ main (int argc, char **argv)
   initbuffer (&line_buf);
 
   /* Initialize the printf format for unnumbered lines. */
-  c = strlen (separator_str);
-  print_no_line_fmt = xmalloc (lineno_width + c + 1);
-  memset (print_no_line_fmt, ' ', lineno_width + c);
-  print_no_line_fmt[lineno_width + c] = '\0';
+  len = strlen (separator_str);
+  print_no_line_fmt = xmalloc (lineno_width + len + 1);
+  memset (print_no_line_fmt, ' ', lineno_width + len);
+  print_no_line_fmt[lineno_width + len] = '\0';
 
   line_no = starting_line_number;
   current_type = body_type;
