@@ -1,5 +1,5 @@
 /* od -- dump files in octal and other formats
-   Copyright (C) 92, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 92, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,38 +17,23 @@
 
 /* Written by Jim Meyering.  */
 
-/* AIX requires this to be the first thing in the file.  */
 #include <config.h>
 
-#ifdef __GNUC__
-#define alloca __builtin_alloca
-#else /* not __GNUC__ */
-#if HAVE_ALLOCA_H
-#include <alloca.h>
-#else /* not HAVE_ALLOCA_H */
-#ifdef _AIX
- #pragma alloca
-#else /* not _AIX */
-#ifdef _WIN32
-#include <malloc.h>
-#include <io.h>
-#else
-char *alloca ();
-#endif /* not _WIN32 */
-#endif /* not _AIX */
-#endif /* not HAVE_ALLOCA_H */
-#endif /* not __GNUC__ */
-
 #include <stdio.h>
 #include <assert.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
-#include "xstrtoul.h"
 #include "error.h"
+#include "xstrtol.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "od"
+
+#define AUTHORS "Jim Meyering"
 
 #if defined(__GNUC__) || defined(STDC_HEADERS)
-#include <float.h>
+# include <float.h>
 #endif
 
 #ifdef HAVE_LONG_DOUBLE
@@ -57,59 +42,10 @@ typedef long double LONG_DOUBLE;
 typedef double LONG_DOUBLE;
 #endif
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-#ifndef SCHAR_MAX
-# define SCHAR_MAX 127
-#endif
-#ifndef SCHAR_MIN
-# define SCHAR_MIN (-128)
-#endif
-#ifndef SHRT_MAX
-# define SHRT_MAX 32767
-#endif
-#ifndef SHRT_MIN
-# define SHRT_MIN (-32768)
-#endif
-#ifndef ULONG_MAX
-# define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
-#endif
-
 #if HAVE_VALUES_H
 # include <values.h>
 #endif
 
-#ifndef BITSPERBYTE
-# define BITSPERBYTE 8
-#endif
-
-#define STREQ(a,b) (strcmp((a), (b)) == 0)
-
-#ifndef MAX
-# define MAX(a, b) ((a) > (b) ? (a) : (b))
-#endif
-
-#ifndef MIN
-# define MIN(a,b) (((a) < (b)) ? (a) : (b))
-#endif
-
-#if defined (_WIN32) && defined (_O_BINARY)
-# define WINDOWS_SETFILEMODE_BINARY(IN_STREAM, FILENAME)		\
-    do									\
-      { /* Turn off DOS text file modes, "rb" doesn't work on stdin.  */\
-	if (_setmode (_fileno ((IN_STREAM)), _O_BINARY) == -1)		\
-          {								\
-	    error (0, errno, "%s", (FILENAME));				\
-	    err = 1;							\
-	    continue;							\
-	  }								\
-      }									\
-    while (0)
-#else
-# define WINDOWS_SETFILEMODE_BINARY(IN_STREAM, FILENAME) /* empty */
-#endif
-
 /* The default number of input bytes per output line.  */
 #define DEFAULT_BYTES_PER_BLOCK 16
 
@@ -128,8 +64,10 @@ typedef double LONG_DOUBLE;
 # define LDBL_DIG DBL_DIG
 #endif
 
-char *xmalloc ();
-char *xrealloc ();
+#if !HAVE_FSEEKO
+# undef fseeko
+# define fseeko(Stream, Offset, Whence) (-1)
+#endif
 
 enum size_spec
   {
@@ -162,6 +100,8 @@ struct tspec
     enum size_spec size;
     void (*print_function) ();
     char *fmt_string;
+    int hexl_mode_trailer;
+    int field_width;
   };
 
 /* The name this program was run with.  */
@@ -240,7 +180,7 @@ static long int pseudo_offset;
 
 /* Function to format an address and optionally an additional parenthesized
    pseudo-address; it returns the formatted string.  */
-static const char *(*format_address) __P ((long unsigned int));
+static const char *(*format_address) PARAMS ((long unsigned int));
 
 /* The number of input bytes to skip before formatting and writing.  */
 static off_t n_bytes_to_skip = 0;
@@ -296,12 +236,6 @@ static enum size_spec integral_type_size[MAX_INTEGRAL_TYPE_SIZE + 1];
 #define MAX_FP_TYPE_SIZE sizeof(LONG_DOUBLE)
 static enum size_spec fp_type_size[MAX_FP_TYPE_SIZE + 1];
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
   /* POSIX options.  */
@@ -315,12 +249,13 @@ static struct option const long_options[] =
   {"strings", optional_argument, NULL, 's'},
   {"traditional", no_argument, NULL, 'B'},
   {"width", optional_argument, NULL, 'w'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -384,11 +319,12 @@ for sizeof(double) or L for sizeof(long double).\n\
 \n\
 RADIX is d for decimal, o for octal, x for hexadecimal or n for none.\n\
 BYTES is hexadecimal with 0x or 0X prefix, it is multiplied by 512\n\
-with b suffix, by 1024 with k and by 1048576 with m.  -s without a\n\
-number implies 3.  -w without a number implies 32.  By default, od\n\
-uses -A o -t d2 -w 16.\n\
+with b suffix, by 1024 with k and by 1048576 with m.  Adding a z suffix to\n\
+any type adds a display of printable characters to the end of each line\n\
+of output.  -s without a number implies 3.  -w without a number implies 32.\n\
+By default, od uses -A o -t d2 -w 16.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -431,7 +367,7 @@ print_s_char (long unsigned int n_bytes, const char *block,
       if (tmp > SCHAR_MAX)
 	tmp -= SCHAR_MAX - SCHAR_MIN + 1;
       assert (tmp <= SCHAR_MAX);
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (unsigned char);
     }
 }
@@ -444,7 +380,7 @@ print_char (long unsigned int n_bytes, const char *block,
   for (i = n_bytes; i > 0; i--)
     {
       unsigned int tmp = *(const unsigned char *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (unsigned char);
     }
 }
@@ -460,10 +396,11 @@ print_s_short (long unsigned int n_bytes, const char *block,
       if (tmp > SHRT_MAX)
 	tmp -= SHRT_MAX - SHRT_MIN + 1;
       assert (tmp <= SHRT_MAX);
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (unsigned short);
     }
 }
+
 static void
 print_short (long unsigned int n_bytes, const char *block,
 	     const char *fmt_string)
@@ -472,7 +409,7 @@ print_short (long unsigned int n_bytes, const char *block,
   for (i = n_bytes / sizeof (unsigned short); i > 0; i--)
     {
       unsigned int tmp = *(const unsigned short *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (unsigned short);
     }
 }
@@ -485,7 +422,7 @@ print_int (long unsigned int n_bytes, const char *block,
   for (i = n_bytes / sizeof (unsigned int); i > 0; i--)
     {
       unsigned int tmp = *(const unsigned int *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (unsigned int);
     }
 }
@@ -498,7 +435,7 @@ print_long (long unsigned int n_bytes, const char *block,
   for (i = n_bytes / sizeof (unsigned long); i > 0; i--)
     {
       unsigned long tmp = *(const unsigned long *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (unsigned long);
     }
 }
@@ -511,7 +448,7 @@ print_float (long unsigned int n_bytes, const char *block,
   for (i = n_bytes / sizeof (float); i > 0; i--)
     {
       float tmp = *(const float *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (float);
     }
 }
@@ -524,7 +461,7 @@ print_double (long unsigned int n_bytes, const char *block,
   for (i = n_bytes / sizeof (double); i > 0; i--)
     {
       double tmp = *(const double *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (double);
     }
 }
@@ -538,13 +475,28 @@ print_long_double (long unsigned int n_bytes, const char *block,
   for (i = n_bytes / sizeof (LONG_DOUBLE); i > 0; i--)
     {
       LONG_DOUBLE tmp = *(const LONG_DOUBLE *) block;
-      printf (fmt_string, tmp, (i == 1 ? '\n' : ' '));
+      printf (fmt_string, tmp);
       block += sizeof (LONG_DOUBLE);
     }
 }
 
 #endif
 
+static void
+dump_hexl_mode_trailer (long unsigned int n_bytes, const char *block)
+{
+  int i;
+  fputs ("  >", stdout);
+  for (i = n_bytes; i > 0; i--)
+    {
+      unsigned int c = *(const unsigned char *) block;
+      unsigned int c2 = (ISPRINT(c) ? c : '.');
+      putchar (c2);
+      block += sizeof (unsigned char);
+    }
+  putchar ('<');
+}
+
 static void
 print_named_ascii (long unsigned int n_bytes, const char *block,
 		   const char *unused_fmt_string)
@@ -567,7 +519,7 @@ print_named_ascii (long unsigned int n_bytes, const char *block,
 	  s = buf;
 	}
 
-      printf ("%3s%c", s, (i == 1 ? '\n' : ' '));
+      printf (" %3s", s);
       block += sizeof (unsigned char);
     }
 }
@@ -622,7 +574,7 @@ print_ascii (long unsigned int n_bytes, const char *block,
 	  s = (const char *) buf;
 	}
 
-      printf ("%3s%c", s, (i == 1 ? '\n' : ' '));
+      printf (" %3s", s);
       block += sizeof (unsigned char);
     }
 }
@@ -681,6 +633,7 @@ decode_one_format (const char *s_orig, const char *s, const char **next,
   void (*print_function) ();
   const char *p;
   unsigned int c;
+  unsigned int field_width = 0;
 
   assert (tspec != NULL);
 
@@ -748,29 +701,29 @@ this system doesn't provide a %lu-byte integral type"), s_orig, size);
 	{
 	case 'd':
 	  fmt = SIGNED_DECIMAL;
-	  sprintf (fmt_string, "%%%u%sd%%c",
-		   bytes_to_signed_dec_digits[size],
+	  sprintf (fmt_string, " %%%u%sd",
+		   (field_width = bytes_to_signed_dec_digits[size]),
 		   (size_spec == LONG ? "l" : ""));
 	  break;
 
 	case 'o':
 	  fmt = OCTAL;
-	  sprintf (fmt_string, "%%0%u%so%%c",
-		   bytes_to_oct_digits[size],
+	  sprintf (fmt_string, " %%0%u%so",
+		   (field_width = bytes_to_oct_digits[size]),
 		   (size_spec == LONG ? "l" : ""));
 	  break;
 
 	case 'u':
 	  fmt = UNSIGNED_DECIMAL;
-	  sprintf (fmt_string, "%%%u%su%%c",
-		   bytes_to_unsigned_dec_digits[size],
+	  sprintf (fmt_string, " %%%u%su",
+		   (field_width = bytes_to_unsigned_dec_digits[size]),
 		   (size_spec == LONG ? "l" : ""));
 	  break;
 
 	case 'x':
 	  fmt = HEXADECIMAL;
-	  sprintf (fmt_string, "%%0%u%sx%%c",
-		   bytes_to_hex_digits[size],
+	  sprintf (fmt_string, " %%0%u%sx",
+		   (field_width = bytes_to_hex_digits[size]),
 		   (size_spec == LONG ? "l" : ""));
 	  break;
 
@@ -858,27 +811,27 @@ this system doesn't provide a %lu-byte floating point type"), s_orig, size);
 	case FLOAT_SINGLE:
 	  print_function = print_float;
 	  /* Don't use %#e; not all systems support it.  */
-	  pre_fmt_string = "%%%d.%de%%c";
+	  pre_fmt_string = " %%%d.%de";
 	  fmt_string = xmalloc (strlen (pre_fmt_string));
 	  sprintf (fmt_string, pre_fmt_string,
-		   FLT_DIG + 8, FLT_DIG);
+		   (field_width = FLT_DIG + 8), FLT_DIG);
 	  break;
 
 	case FLOAT_DOUBLE:
 	  print_function = print_double;
-	  pre_fmt_string = "%%%d.%de%%c";
+	  pre_fmt_string = " %%%d.%de";
 	  fmt_string = xmalloc (strlen (pre_fmt_string));
 	  sprintf (fmt_string, pre_fmt_string,
-		   DBL_DIG + 8, DBL_DIG);
+		   (field_width = DBL_DIG + 8), DBL_DIG);
 	  break;
 
 #ifdef HAVE_LONG_DOUBLE
 	case FLOAT_LONG_DOUBLE:
 	  print_function = print_long_double;
-	  pre_fmt_string = "%%%d.%dLe%%c";
+	  pre_fmt_string = " %%%d.%dLe";
 	  fmt_string = xmalloc (strlen (pre_fmt_string));
 	  sprintf (fmt_string, pre_fmt_string,
-		   LDBL_DIG + 8, LDBL_DIG);
+		   (field_width = LDBL_DIG + 8), LDBL_DIG);
 	  break;
 #endif
 
@@ -893,6 +846,7 @@ this system doesn't provide a %lu-byte floating point type"), s_orig, size);
       size_spec = CHAR;
       fmt_string = NULL;
       print_function = print_named_ascii;
+      field_width = 3;
       break;
 
     case 'c':
@@ -901,6 +855,7 @@ this system doesn't provide a %lu-byte floating point type"), s_orig, size);
       size_spec = CHAR;
       fmt_string = NULL;
       print_function = print_ascii;
+      field_width = 3;
       break;
 
     default:
@@ -914,6 +869,11 @@ this system doesn't provide a %lu-byte floating point type"), s_orig, size);
   tspec->print_function = print_function;
   tspec->fmt_string = fmt_string;
 
+  tspec->field_width = field_width;
+  tspec->hexl_mode_trailer = (*s == 'z');
+  if (tspec->hexl_mode_trailer)
+    s++;
+
   if (next != NULL)
     *next = s;
 
@@ -944,8 +904,9 @@ decode_format_string (const char *s)
       if (n_specs >= n_specs_allocated)
 	{
 	  n_specs_allocated = 1 + (3 * n_specs_allocated) / 2;
-	  spec = (struct tspec *) xrealloc (spec, (n_specs_allocated
-						   * sizeof (struct tspec)));
+	  spec = (struct tspec *) xrealloc ((char *) spec,
+					    (n_specs_allocated
+					     * sizeof (struct tspec)));
 	}
 
       memcpy ((char *) &spec[n_specs], (char *) &tspec,
@@ -992,7 +953,7 @@ skip (off_t n_skip)
 	      continue;
 	    }
 	}
-      WINDOWS_SETFILEMODE_BINARY (in_stream, input_filename);
+      SET_BINARY (fileno (in_stream));
 
       if (n_skip == 0)
 	break;
@@ -1033,13 +994,10 @@ skip (off_t n_skip)
 	    }
 	  else
 	    {
-	      /* fseek may work on some streams for which lseek doesn't.
-		 But fseek's offset argument is restricted to the range
-		 of type `long'.  So if N_SKIP is too large or if fseek
-		 fails, try lseek.  */
-	      if ((n_skip <= LONG_MAX
-		   && fseek (in_stream, (long) n_skip, SEEK_SET) == 0)
-		  || lseek (fileno (in_stream), n_skip, SEEK_SET) >= 0)
+	      /* Try fseeko if available, fseek otherwise.  */
+	      if (fseeko (in_stream, n_skip, SEEK_SET) == 0
+		  || (n_skip <= LONG_MAX
+		      && fseek (in_stream, (long) n_skip, SEEK_SET) == 0))
 		{
 		  n_skip = 0;
 		  break;
@@ -1143,10 +1101,22 @@ write_block (long unsigned int current_offset, long unsigned int n_bytes,
       prev_pair_equal = 0;
       for (i = 0; i < n_specs; i++)
 	{
-	  printf ("%s ", (i == 0
-			  ? format_address (current_offset)
-			  : address_pad));
+	  const char *addr_or_pad = (i == 0
+				     ? format_address (current_offset)
+				     : address_pad);
+
+	  fputs (addr_or_pad, stdout);
 	  (*spec[i].print_function) (n_bytes, curr_block, spec[i].fmt_string);
+	  if (spec[i].hexl_mode_trailer)
+	    {
+	      /* space-pad out to full line width, then dump the trailer */
+	      int datum_width = width_bytes[spec[i].size];
+	      int blank_fields = (bytes_per_block - n_bytes) / datum_width;
+	      int field_width = spec[i].field_width + 1;
+	      printf ("%*s", blank_fields * field_width, "");
+	      dump_hexl_mode_trailer (n_bytes, curr_block);
+	    }
+	  putchar ('\n');
 	}
     }
   first = 0;
@@ -1241,7 +1211,7 @@ read_char (int *c)
 		  err = 1;
 		}
 	    }
-	  WINDOWS_SETFILEMODE_BINARY (in_stream, input_filename);
+	  SET_BINARY (fileno (in_stream));
 	}
       while (in_stream == NULL);
     }
@@ -1314,7 +1284,7 @@ read_block (size_t n, char *block, size_t *n_bytes_in_buffer)
 		  err = 1;
 		}
 	    }
-	  WINDOWS_SETFILEMODE_BINARY (in_stream, input_filename);
+	  SET_BINARY (fileno (in_stream));
 	}
       while (in_stream == NULL);
     }
@@ -1498,7 +1468,8 @@ dump_strings (void)
     tryline:
 
       if (limit_bytes_to_format
-	  && address >= (n_bytes_to_skip + max_bytes_to_format - string_min))
+	  && address >= (n_bytes_to_skip + max_bytes_to_format -
+			 (off_t) string_min))
 	break;
 
       for (i = 0; i < string_min; i++)
@@ -1651,8 +1622,7 @@ main (int argc, char **argv)
   flag_dump_strings = 0;
 
   while ((c = getopt_long (argc, argv, "abcdfhilos::xw::A:j:N:t:v",
-			   long_options, (int *) 0))
-	 != EOF)
+			   long_options, NULL)) != -1)
     {
       unsigned long int tmp;
       enum strtol_error s_err;
@@ -1782,6 +1752,10 @@ the maximum\nrepresentable value of type `long'"), optarg);
 	    }
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	  break;
@@ -1791,15 +1765,6 @@ the maximum\nrepresentable value of type `long'"), optarg);
   if (n_failed_decodes > 0)
     exit (EXIT_FAILURE);
 
-  if (show_version)
-    {
-      printf ("od (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (flag_dump_strings && n_specs > 0)
     error (EXIT_FAILURE, 0,
 	   _("no type may be specified when dumping strings"));
@@ -1868,14 +1833,14 @@ the maximum\nrepresentable value of type `long'"), optarg);
 	  else
 	    {
 	      error (0, 0,
-	    _("in compatibility mode the last 2 arguments must be offsets"));
+	    _("in compatibility mode, the last two arguments must be offsets"));
 	      usage (1);
 	    }
 	}
-      else
+      else if (n_files > 3)
 	{
 	  error (0, 0,
-	   _("in compatibility mode there may be no more than 3 arguments"));
+		 _("compatibility mode supports at most three arguments"));
 	  usage (1);
 	}
 
