@@ -1,5 +1,5 @@
 /* paste - merge lines of files
-   Copyright (C) 1984 by David M. Ihnat
+   Copyright (C) 1984, 1997, 1998, 1999 by David M. Ihnat
 
    This program is a total rewrite of the Bell Laboratories Unix(Tm)
    command of the same name, as of System V.  It contains no proprietary
@@ -46,15 +46,17 @@
 #include "system.h"
 #include "error.h"
 
-char *xmalloc ();
-char *xrealloc ();
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "paste"
+
+#define AUTHORS "David M. Ihnat"
 
 /* Indicates that no delimiter should be added in the current position. */
 #define EMPTY_DELIM '\0'
 
 static FILE dummy_closed;
 /* Element marking a file that has reached EOF and been closed. */
-#define	CLOSED (&dummy_closed)
+#define CLOSED (&dummy_closed)
 
 static FILE dummy_endlist;
 /* Element marking end of list of open files. */
@@ -76,18 +78,12 @@ static char *delims;
 /* A pointer to the character after the end of `delims'. */
 static char *delim_end;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
   {"serial", no_argument, 0, 's'},
   {"delimiters", required_argument, 0, 'd'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {0, 0, 0, 0}
 };
 
@@ -194,10 +190,10 @@ paste_parallel (int nfiles, char **fnamptr)
 	{
 	  file_list_size += 12;
 	  delbuf = (char *) xrealloc (delbuf, file_list_size + 2);
-	  fileptr = (FILE **) xrealloc (fileptr, (file_list_size + 1)
+	  fileptr = (FILE **) xrealloc ((char *) fileptr, (file_list_size + 1)
 					* sizeof (FILE *));
 	}
-      if (!strcmp (fnamptr[files_open], "-"))
+      if (STREQ (fnamptr[files_open], "-"))
 	{
 	  have_read_stdin = 1;
 	  fileptr[files_open] = stdin;
@@ -335,7 +331,7 @@ paste_serial (int nfiles, char **fnamptr)
 
   for (; nfiles; nfiles--, fnamptr++)
     {
-      if (!strcmp (*fnamptr, "-"))
+      if (STREQ (*fnamptr, "-"))
 	{
 	  have_read_stdin = 1;
 	  fileptr = stdin;
@@ -402,7 +398,7 @@ paste_serial (int nfiles, char **fnamptr)
   return errors;
 }
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -425,7 +421,7 @@ With no FILE, or when FILE is -, read standard input.\n\
       --version           output version information and exit\n\
 \n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -447,8 +443,7 @@ main (int argc, char **argv)
   strcpy (delims, "\t");
   strcpy (zero_delims, "\\0");
 
-  while ((optc = getopt_long (argc, argv, "d:s", longopts, (int *) 0))
-	 != EOF)
+  while ((optc = getopt_long (argc, argv, "d:s", longopts, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -466,20 +461,15 @@ main (int argc, char **argv)
 	  serial_merge++;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("paste (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (optind == argc)
     argv[argc++] = "-";
 
