@@ -1,5 +1,5 @@
 /* sum -- checksum and count the blocks in a file
-   Copyright (C) 86, 89, 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 86, 89, 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,8 +26,12 @@
 #include <getopt.h>
 #include "system.h"
 #include "error.h"
+#include "safe-read.h"
 
-int safe_read ();
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "sum"
+
+#define AUTHORS "Kayvan Aghaiepour and David MacKenzie"
 
 /* The name this program was run with. */
 char *program_name;
@@ -38,21 +42,15 @@ static int have_read_stdin;
 /* Right-rotate 32-bit integer variable C. */
 #define ROTATE_RIGHT(c) if ((c) & 01) (c) = ((c) >>1) + 0x8000; else (c) >>= 1;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
   {"sysv", no_argument, NULL, 's'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -74,7 +72,7 @@ Print checksum and block counts for each FILE.\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -93,7 +91,7 @@ bsd_sum_file (const char *file, int print_name)
   register long total_bytes = 0; /* The number of bytes. */
   register int ch;		/* Each character read. */
 
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     {
       fp = stdin;
       have_read_stdin = 1;
@@ -107,6 +105,8 @@ bsd_sum_file (const char *file, int print_name)
 	  return -1;
 	}
     }
+  /* Need binary I/O, or else byte counts and checksums are incorrect.  */
+  SET_BINARY (fileno(fp));
 
   while ((ch = getc (fp)) != EOF)
     {
@@ -119,12 +119,12 @@ bsd_sum_file (const char *file, int print_name)
   if (ferror (fp))
     {
       error (0, errno, "%s", file);
-      if (strcmp (file, "-"))
+      if (!STREQ (file, "-"))
 	fclose (fp);
       return -1;
     }
 
-  if (strcmp (file, "-") && fclose (fp) == EOF)
+  if (!STREQ (file, "-") && fclose (fp) == EOF)
     {
       error (0, errno, "%s", file);
       return -1;
@@ -152,7 +152,7 @@ sysv_sum_file (const char *file, int print_name)
   register unsigned long checksum = 0;
   long total_bytes = 0;
 
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     {
       fd = 0;
       have_read_stdin = 1;
@@ -166,6 +166,8 @@ sysv_sum_file (const char *file, int print_name)
 	  return -1;
 	}
     }
+  /* Need binary I/O, or else byte counts and checksums are incorrect.  */
+  SET_BINARY (fd);
 
   while ((bytes_read = safe_read (fd, buf, sizeof buf)) > 0)
     {
@@ -179,12 +181,12 @@ sysv_sum_file (const char *file, int print_name)
   if (bytes_read < 0)
     {
       error (0, errno, "%s", file);
-      if (strcmp (file, "-"))
+      if (!STREQ (file, "-"))
 	close (fd);
       return -1;
     }
 
-  if (strcmp (file, "-") && close (fd) == -1)
+  if (!STREQ (file, "-") && close (fd) == -1)
     {
       error (0, errno, "%s", file);
       return -1;
@@ -213,7 +215,7 @@ main (int argc, char **argv)
 
   have_read_stdin = 0;
 
-  while ((optc = getopt_long (argc, argv, "rs", longopts, (int *) 0)) != -1)
+  while ((optc = getopt_long (argc, argv, "rs", longopts, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -228,20 +230,15 @@ main (int argc, char **argv)
 	  sum_func = sysv_sum_file;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("sum (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   files_given = argc - optind;
   if (files_given == 0)
     {
@@ -257,4 +254,3 @@ main (int argc, char **argv)
     error (EXIT_FAILURE, errno, "-");
   exit (errors == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
-
