@@ -1,5 +1,5 @@
 /* tr -- a filter to translate characters
-   Copyright (C) 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 91, 1995-1998, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,46 +15,24 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-/* Written by Jim Meyering, meyering@cs.utexas.edu.  */
+/* Written by Jim Meyering */
 
 #include <config.h>
 
-/* Get isblank from GNU libc.  */
-#define _GNU_SOURCE
-
 #include <stdio.h>
-#define NDEBUG 1
 #include <assert.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <getopt.h>
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
 #include "system.h"
 #include "error.h"
+#include "safe-read.h"
 
-#ifndef ULONG_MAX
-#define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
-#endif
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "tr"
 
-#ifndef LONG_MAX
-#define LONG_MAX ((long int) (ULONG_MAX >> 1))
-#endif
-
-#ifndef UINT_MAX
-# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
-#endif
-
-#ifndef INT_MAX
-# define INT_MAX ((int) (UINT_MAX >> 1))
-#endif
-
-#ifndef UCHAR_MAX
-#define UCHAR_MAX 0xFF
-#endif
+#define AUTHORS "Jim Meyering"
 
 #define N_CHARS (UCHAR_MAX + 1)
 
@@ -229,11 +207,6 @@ struct E_string
    and is not escaped itself.  */
 #define ES_MATCH(ES, I, C) ((ES)->s[(I)] == (C) && !(ES)->escaped[(I)])
 
-
-char *xmalloc ();
-char *stpcpy ();
-int safe_read ();
-
 /* The name by which this program was run.  */
 char *program_name;
 
@@ -298,7 +271,7 @@ static int truncate_set1 = 0;
 static int translating;
 
 #ifndef BUFSIZ
-#define BUFSIZ 8192
+# define BUFSIZ 8192
 #endif
 
 #define IO_BUF_SIZE BUFSIZ
@@ -330,24 +303,18 @@ static SET_TYPE in_delete_set[N_CHARS];
    two specification strings and the delete switch is not given.  */
 static char xlate[N_CHARS];
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
   {"complement", no_argument, NULL, 'c'},
   {"delete", no_argument, NULL, 'd'},
   {"squeeze-repeats", no_argument, NULL, 's'},
   {"truncate-set1", no_argument, NULL, 't'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -413,7 +380,7 @@ only be used in pairs to specify case conversion.  -s uses SET1 if not\n\
 translating nor deleting; else squeezing uses SET2 and occurs after\n\
 translation or deletion.\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -450,7 +417,7 @@ is_char_class_member (enum Char_class char_class, unsigned int c)
       result = ISCNTRL (c);
       break;
     case CC_DIGIT:
-      result = ISDIGIT (c);
+      result = ISDIGIT_LOCALE (c);
       break;
     case CC_GRAPH:
       result = ISGRAPH (c);
@@ -1837,8 +1804,7 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while ((c = getopt_long (argc, argv, "cdst", long_options,
-			   (int *) 0)) != EOF)
+  while ((c = getopt_long (argc, argv, "cdst", long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -1861,21 +1827,16 @@ main (int argc, char **argv)
 	  truncate_set1 = 1;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (2);
 	  break;
 	}
     }
 
-  if (show_version)
-    {
-      printf ("tr (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   posix_pedantic = (getenv ("POSIXLY_CORRECT") != NULL);
 
   non_option_args = argc - optind;
@@ -1933,6 +1894,11 @@ without squeezing repeats"));
 
   validate (s1, s2);
 
+  /* Use binary I/O, since `tr' is sometimes used to transliterate
+     non-printable characters, or characters which are stripped away
+     by text-mode reads (like CR and ^Z).  */
+  SET_BINARY2 (STDIN_FILENO, STDOUT_FILENO);
+
   if (squeeze_repeats && non_option_args == 1)
     {
       set_initialize (s1, complement, in_squeeze_set);
@@ -2034,7 +2000,7 @@ construct in string1 must be aligned with a corresponding construct\n\
 	      else
 		{
 		  /* The following should have been checked by validate...  */
-		  if (c2 == -1)
+		  if (c1 == -1 || c2 == -1)
 		    break;
 		  xlate[c1] = c2;
 		}
