@@ -1,5 +1,5 @@
 /* wc - print the number of bytes, words, and lines in files
-   Copyright (C) 85, 91, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 85, 91, 1995-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,33 +12,45 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Paul Rubin, phr@ocf.berkeley.edu
    and David MacKenzie, djm@gnu.ai.mit.edu. */
 
 #include <config.h>
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
 
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
 #include "error.h"
+#include "human.h"
+#include "safe-read.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "wc"
+
+#define AUTHORS "Paul Rubin and David MacKenzie"
 
 /* Size of atomic reads. */
 #define BUFFER_SIZE (16 * 1024)
 
-int safe_read ();
-
 /* The name this program was run with. */
 char *program_name;
 
-/* Cumulative number of lines, words, and chars in all files so far. */
-static unsigned long total_lines, total_words, total_chars;
+/* Cumulative number of lines, words, and chars in all files so far.
+   max_line_length is the maximum over all files processed so far.  */
+static uintmax_t total_lines;
+static uintmax_t total_words;
+static uintmax_t total_chars;
+static uintmax_t max_line_length;
 
 /* Which counts to print. */
-static int print_lines, print_words, print_chars;
+static int print_lines, print_words, print_chars, print_linelength;
 
 /* Nonzero if we have ever read the standard input. */
 static int have_read_stdin;
@@ -46,11 +58,9 @@ static int have_read_stdin;
 /* The error code to return to the system. */
 static int exit_status;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exits.  */
-static int show_version;
+/* If nonzero, do not line up columns but instead separate numbers by
+   a single space as specified in Single Unix Specification and POSIX. */
+static int posixly_correct;
 
 static struct option const longopts[] =
 {
@@ -58,12 +68,13 @@ static struct option const longopts[] =
   {"chars", no_argument, NULL, 'c'},
   {"lines", no_argument, NULL, 'l'},
   {"words", no_argument, NULL, 'w'},
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {"max-line-length", no_argument, NULL, 'L'},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -81,32 +92,46 @@ more than one FILE is specified.  With no FILE, or when FILE is -,\n\
 read standard input.\n\
   -c, --bytes, --chars   print the byte counts\n\
   -l, --lines            print the newline counts\n\
+  -L, --max-line-length  print the length of the longest line\n\
   -w, --words            print the word counts\n\
       --help             display this help and exit\n\
       --version          output version information and exit\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static void
-write_counts (long unsigned int lines, long unsigned int words,
-	      long unsigned int chars, const char *file)
+write_counts (uintmax_t lines,
+	      uintmax_t words,
+	      uintmax_t chars,
+	      uintmax_t linelength,
+	      const char *file)
 {
+  char buf[LONGEST_HUMAN_READABLE + 1];
+  char const *space = "";
+  char const *format_int = (posixly_correct ? "%s" : "%7s");
+  char const *format_sp_int = (posixly_correct ? "%s%s" : "%s%7s");
+
   if (print_lines)
-    printf ("%7lu", lines);
+    {
+      printf (format_int, human_readable (lines, buf, 1, 1));
+      space = " ";
+    }
   if (print_words)
     {
-      if (print_lines)
-	putchar (' ');
-      printf ("%7lu", words);
+      printf (format_sp_int, space, human_readable (words, buf, 1, 1));
+      space = " ";
     }
   if (print_chars)
     {
-      if (print_lines || print_words)
-	putchar (' ');
-      printf ("%7lu", chars);
+      printf (format_sp_int, space, human_readable (chars, buf, 1, 1));
+      space = " ";
+    }
+  if (print_linelength)
+    {
+      printf (format_sp_int, space, human_readable (linelength, buf, 1, 1));
     }
   if (*file)
     printf (" %s", file);
@@ -117,11 +142,14 @@ static void
 wc (int fd, const char *file)
 {
   char buf[BUFFER_SIZE + 1];
-  register int bytes_read;
-  register int in_word = 0;
-  register unsigned long lines, words, chars;
+  ssize_t bytes_read;
+  int in_word = 0;
+  uintmax_t lines, words, chars, linelength;
 
-  lines = words = chars = 0;
+  lines = words = chars = linelength = 0;
+
+  /* We need binary input, since `wc' relies on `lseek' and byte counts.  */
+  SET_BINARY (fd);
 
   /* When counting only bytes, save some line- and word-counting
      overhead.  If FD is a `regular' Unix file, using lseek is enough
@@ -133,7 +161,7 @@ wc (int fd, const char *file)
      `(dd ibs=99k skip=1 count=0; ./wc -c) < /etc/group'
      should make wc report `0' bytes.  */
 
-  if (print_chars && !print_words && !print_lines)
+  if (print_chars && !print_words && !print_lines && !print_linelength)
     {
       off_t current_pos, end_pos;
       struct stat stats;
@@ -160,7 +188,7 @@ wc (int fd, const char *file)
 	    }
 	}
     }
-  else if (!print_words)
+  else if (!print_words && !print_linelength)
     {
       /* Use a separate loop when counting only lines or lines and bytes --
 	 but not words.  */
@@ -183,9 +211,11 @@ wc (int fd, const char *file)
     }
   else
     {
+      uintmax_t linepos = 0;
+
       while ((bytes_read = safe_read (fd, buf, BUFFER_SIZE)) > 0)
 	{
-	  register char *p = buf;
+	  const char *p = buf;
 
 	  chars += bytes_read;
 	  do
@@ -197,9 +227,18 @@ wc (int fd, const char *file)
 		  /* Fall through. */
 		case '\r':
 		case '\f':
+		  if (linepos > linelength)
+		    linelength = linepos;
+		  linepos = 0;
+		  goto word_separator;
 		case '\t':
-		case '\v':
+		  linepos += 8 - (linepos % 8);
+		  goto word_separator;
 		case ' ':
+		  linepos++;
+		  /* Fall through. */
+		case '\v':
+		word_separator:
 		  if (in_word)
 		    {
 		      in_word = 0;
@@ -207,6 +246,7 @@ wc (int fd, const char *file)
 		    }
 		  break;
 		default:
+		  linepos++;
 		  in_word = 1;
 		  break;
 		}
@@ -218,20 +258,24 @@ wc (int fd, const char *file)
 	  error (0, errno, "%s", file);
 	  exit_status = 1;
 	}
+      if (linepos > linelength)
+	linelength = linepos;
       if (in_word)
 	words++;
     }
 
-  write_counts (lines, words, chars, file);
+  write_counts (lines, words, chars, linelength, file);
   total_lines += lines;
   total_words += words;
   total_chars += chars;
+  if (linelength > max_line_length)
+    max_line_length = linelength;
 }
 
 static void
 wc_file (const char *file)
 {
-  if (!strcmp (file, "-"))
+  if (STREQ (file, "-"))
     {
       have_read_stdin = 1;
       wc (0, file);
@@ -266,10 +310,11 @@ main (int argc, char **argv)
   textdomain (PACKAGE);
 
   exit_status = 0;
-  print_lines = print_words = print_chars = 0;
-  total_lines = total_words = total_chars = 0;
+  posixly_correct = (getenv ("POSIXLY_CORRECT") != NULL);
+  print_lines = print_words = print_chars = print_linelength = 0;
+  total_lines = total_words = total_chars = max_line_length = 0;
 
-  while ((optc = getopt_long (argc, argv, "clw", longopts, (int *) 0)) != EOF)
+  while ((optc = getopt_long (argc, argv, "clLw", longopts, NULL)) != -1)
     switch (optc)
       {
       case 0:
@@ -287,20 +332,19 @@ main (int argc, char **argv)
 	print_words = 1;
 	break;
 
+      case 'L':
+	print_linelength = 1;
+	break;
+
+      case_GETOPT_HELP_CHAR;
+
+      case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
       default:
 	usage (1);
       }
 
-  if (show_version)
-    {
-      printf ("wc (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
-  if (print_lines + print_words + print_chars == 0)
+  if (print_lines + print_words + print_chars + print_linelength == 0)
     print_lines = print_words = print_chars = 1;
 
   nfiles = argc - optind;
@@ -316,7 +360,8 @@ main (int argc, char **argv)
 	wc_file (argv[optind]);
 
       if (nfiles > 1)
-	write_counts (total_lines, total_words, total_chars, _("total"));
+	write_counts (total_lines, total_words, total_chars, max_line_length,
+		      _("total"));
     }
 
   if (have_read_stdin && close (0))
